<?php

/*
 * This file is Copyright (c) 2005 by Martijn van der Kwast <mvdkwast@gmx.net>
 * and part of the Coppermine Integration plugin for WordPress.
 */

require_once('../../../wp-blog-header.php');
require_once('./coppermine.php');

/* do nothing for users who are not logged in. */
$login_ok = false;

if (!$user_ID) {
	get_currentuserinfo();
}

if (!$user_ID) {
	html_error_page('You need to be logged in to use the image selecter.');
	exit;
}	

/* connect */
cpg_connect();

/* check if user level is sufficient */


if (function_exists('get_role')) {
	// check user level for wp20
	$user = new WP_User($user_ID);
	if (!$user->has_cap('cpg_display_inserter')) {
		html_error_page('You don\'t have the user level required to access this page.');
		exit;
	}
} else {
	// check user level for wp15
	$user=get_userdata($user_ID);
	$min_level = get_option('cpg_min_level');

	if (!is_numeric($min_level)) {
		html_error_page('Minimum user level not set. Run configuration page first.'.$min_level);
		exit;
	}

	if ($user->user_level < $min_level ) {
		html_error_page('You don\'t have the user level required to access this page.');
		exit;
	}
}


/* prepare AJAX and display page */

$rpcurl=get_settings('siteurl').'/wp-content/plugins/coppermine/coppermine-server.php';

html_template();

/******************************************************************************/
/*							     LAYOUT FUNCTIONS                             */
/******************************************************************************/

function html_head()
{
	global $rpcurl;
?>
	<title>Image Selecter</title>
	<link rel="stylesheet" href="coppermine-browser.css" type="text/css" />
	<link rel="stylesheet" href="dtree.css" type="text/css" />

	<script type="text/javascript" src="dtree.js"></script>
	<script type="text/javascript" src="coppermine-browser.js"></script>
	<script type="text/javascript">
		var cpg_rpcurl='<?echo $rpcurl ?>';
	</script>

	<?php _cpg_image_inserter_head(); ?>
<?php
}

function html_header()
{
	_cpg_image_insert_dropdowns();
}

// escape quotes and replace newlines with <br/>
function quotequotes($str)
{
	return str_replace(array("'","\n","\r"), array('&#039;','<br/>',''),$str);
}

function html_navtree()
{
	global $user_ID;

	if (get_option('cpg_integrate_users')) {
		$wp_user = get_userdata($user_ID);
		if ($wp_user) {
			$cpg_user = cpg_get_login_info($wp_user->user_login);
		}
	}

?>

<div class="dtree">
        <script type="text/javascript">
			// <![CDATA[
			// <!--
				d = new dTree('d');

<?php
	print "d.add(0,-1,'Gallery','','desc','','img/base.gif');";

	/* HACK: let's use big values and hope nobody has more than 99999+albums, users or whatever */
	$arbitrary_offset=100000;

	if (get_option('cpg_integrate_users')) {
		/* Multi user: insert current user's stuff followed by other users, followed by categories view */

		/* insert my albums */
		$icon="'img/star.gif'";
		print "d.add(1,0,'My Albums','','desc','',$icon,$icon,1);";
		$albums = cpg_db_get_albums($cpg_user->user_id);
		if ($albums) foreach($albums as $album) {
			$title=quotequotes($album->title);
			$description=quotequotes($album->description);
			print "d.add($arbitrary_offset+$album->aid, 1, '$title', 'javascript:openAlbum($album->aid)', '$description', '', 'img/icons.gif');";
		}

		$arbitrary_offset+=100000;
		
		$icon="'img/people.gif'";
		$picon="'img/person.gif'"; // picon, c'est bon

		/* insert other users albums */
		print "d.add(2,0,'Other Users','','Other Users','',$icon,$icon);";
		$users = cpg_db_get_all_userdata();
		if ($users) foreach($users as $user) {
			if ($cpg_user && $user->user_id==$cpg_user->user_id) {
				continue;
			}
			$iid = $arbitrary_offset+$user->user_id;
			$name=quotequotes($user->user_name);
			print "d.add($iid,2,'$name','','a User','',$picon,$picon);";

			$albums = cpg_db_get_albums($user->user_id);
			if ($albums) foreach ($albums as $album) {
				if (cpg_can_access_album($cpg_user->user_id,$album->aid)) {
					$iaid = $arbitrary_offset+$user->user_id*10000+$album->aid;
					$title=quotequotes($album->title);
					$description=quotequotes($album->description);
					print "d.add($iaid, $iid, '$title', 'javascript:openAlbum($album->aid)', '$description', '', 'img/icons.gif');";
				}
			}
		}

		/* insert categories */

		
	}
	else {
		/* Single user: insert all albums, don't care about who created them */
		$albums = cpg_db_get_albums(0);
		if ($albums) foreach ($albums as $album) {
			$title=quotequotes($album->title);
			$description=quotequotes($album->description);
			print "d.add($album->aid, 0, '$title', 'javascript:openAlbum($album->aid)', '$description', '', 'img/icons.gif');";
		}
	}
				
?>
        	document.write(d);
        // -->
		// ]]>
        </script>


</div>

<?php
}

function html_sidebar()
{
	html_navtree();
}

function html_mainview()
{
	help_text('');
}

function html_template()
{
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
	<head>
		<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
		<?php html_head(); ?>
	</head>
	<body>
		<div id="headerbox"><?php html_header(); ?></div>
		<div id="contentbox">	
			<div id="leftbox"><?php html_sidebar(); ?> </div>
			<div id="mainbox"><?php html_mainview(); ?> </div>
		</div>
	</body>
</html>

<?php
}


/* Introduction text */
function help_text($str)
{
?>
<div class="instructions">
<h2>Image Chooser</h2>
<p>This window allows you to choose pictures from a Coppermine gallery to insert in your WordPress posts.</p>
<p>Use the treeview in the left pane to navigate the image database. </p>
<p>Click on the pictures and album titles in the right pane to insert them in your post. </p>
<p>The dropdown boxes in the upper part allow you to control the position and the size of the images in your post. </p>
<p>For comments and bug reports, please visit <a href="javascript:void(window.open('http://www.stilglog.com/wordpress-plugins/coppermine-plugin/'));" title="Plugin page on Stilglog">this</a> page.</p>
</div>
<?php
}

function html_error_page($str)
{
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
	<head>
		<title>Image Selecter &raquo; Login Error</title>
		<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
	</head>
	<body>
		<div id="errorbox">
			<?php echo $str; ?>
		</div>
	</body>
</html>
<?php
}


?>
