<?php
/*
 * This file is Copyright (c) 2005 by Martijn van der Kwast <mvdkwast@gmx.net>
 * and part of the Coppermine Integration plugin for WordPress.
 */

/******************************************************************************/
/*				   GENERIC SETTINGS MANAGER & FORM GENERATOR                  */
/******************************************************************************/

$settings_version = '3.5';

/*----------------------------------------------------------------------------*/
/*							  SETTING MANIPULATION                            */
/*----------------------------------------------------------------------------*/

/* return true iff an installation is found */
function settings_are_installed()
{
	return get_option('cpg_options_version');
}

/* write default values for parameters to options table */
function settings_install()
{
	global $cpg_options;

	foreach ($cpg_options as $option) {
		add_option( $option[0], $option[2], $option[3] );
	}
}

/* add settings that don't exist yet */
function settings_update()
{
	global $cpg_options;

	foreach ($cpg_options as $option) {
		if (false===get_option($option[0])) {
			add_option( $option[0], $option[2], $option[3] );
		}
	}
}


/* remove all settings from database, is this script polite or what ? */
function settings_uninstall()
{
	global $cpg_options;

	foreach ($cpg_options as $option) {
		delete_option($option[0]);
	}
}

/* reset settings to defaults */
function settings_reset()
{
	global $cpg_options;
	
	foreach ($cpg_options as $option) {
		delete_option($option[0]);
		add_option( $option[0], $option[2], $option[3] );
	}
}

/* save all settings to database from $_POST */
function settings_save()
{
	global $cpg_options;

	foreach ($cpg_options as $option) {
		if (isset($_POST[$option[0]])) {
			update_option($option[0], $_POST[$option[0]]);
		}
	}
}


/*----------------------------------------------------------------------------*/
								   /* Form */
/*----------------------------------------------------------------------------*/

function form_elt_user_level($option)
{
	$value = get_option($option['0']);
	
	print "<select id=\"$option[0]\" name=\"$option[0]\">";

	if ($value==0) {
		print '<option value="0" selected="selected">Everybody</option>';
	} else {
		print '<option value="0">Everybody</option>';
	}
		
	for ($i=1;$i<11;$i++) {
		$selected = ($value==$i) ? ' selected="selected"' : '';
		print "<option value=\"$i\" $selected>$i</option>";
	}
		
	if ($value>10) {
		print '<option value="0" selected="selected">Nobody</option>';
	} else {
		print '<option value="0">Nobody</option>';
	}
	
	print '</select>';
}

// hash: key => label
function _form_elt_dropdown($option, $hash)
{
	print "<select id=\"$option[0]\" name=\"$option[0]\">";
	$default = get_option($option[0]);

	foreach ($hash as $k => $v) {
		$selected = ($default==$k) ? ' selected="selected"' : '';
		print "<option value=\"$k\" $selected>$v</option>";
	}

	print '</select>';
}

// hash: key => array( label, ..,  .. )

function _form_elt_dropdown_arr($option, $hash)
{
	print "<select id=\"$option[0]\" name=\"$option[0]\">";
	$default = get_option($option[0]);

	foreach ($hash as $k => $v) {
		$selected = ($default==$k) ? ' selected="selected"' : '';
		print "<option value=\"$k\" $selected>$v[0]</option>";
	}

	print '</select>';
}

function form_elt_link_style($option)
{
	global $cpg_link_styles;
	_form_elt_dropdown_arr($option, $cpg_link_styles);
}


function form_elt_charset($option)
{
	$charsets = array(
		'windows-1256' => 'Arabic' ,
		'iso-8859-4' => 'Baltic' ,
		'iso-8859-2' => 'Central European' ,
		'euc-cn' => 'Chinese Simplified' ,
		'big5' => 'Chinese Traditional' ,
		'koi8-r' => 'Cyrillic' ,
		'iso-8859-7' => 'Greek' ,
		'iso-8859-8-i' => 'Hebrew' ,
		'x-mac-icelandic' => 'Icelandic' ,
		'euc-jp' => 'Japanese' ,
		'euc-kr' => 'Korean' ,
		'iso-8859-3' => 'Maltese' ,
		'windows-874 ' => 'Thai' ,
		'iso-8859-9' => 'Turkish' ,
		'utf-8' => 'Unicode' ,
		'windows-1258' => 'Vietnamese' ,
		'iso-8859-1' => 'Western' ,
	);

	_form_elt_dropdown($option, $charsets);
}

function form_elt_user_group($option)
{
	global $cpg_cmdb;

	if ($cpg_cmdb==false || $cpg_cmdb->unavailable) {
		form_elt_int($option);
	} else {
		$groups=array();
		$dbgroups = @cpg_db_get_groups();
		if ($dbgroups) {
			foreach($dbgroups as $dbgroup) {
				$groups[$dbgroup->group_id] = $dbgroup->group_name;
			}
			_form_elt_dropdown($option, $groups);
		} else {
			form_elt_int($option);
		}
	}
}



function form_elt_int($option)
{
	$val=get_option($option[0]);
	print "<input id=\"$option[0]\" name=\"$option[0]\" type=\"text\" value=\"$val\" />";
}

function form_elt_bool($option)
{
	$val=get_option($option[0]);
	if ($val) {
		print "<input id=\"$option[0]\" type=\"radio\" name=\"$option[0]\" value=\"1\" checked=\"checked\">Yes</input>";
		print "<input id=\"$option[0]-no\" type=\"radio\" name=\"$option[0]\" value=\"0\">No</input>";
	} else {
		print "<input id=\"$option[0]\" type=\"radio\" name=\"$option[0]\" value=\"1\">Yes</input>";
		print "<input id=\"$option[0]-no\" type=\"radio\" name=\"$option[0]\" value=\"0\" checked=\"checked\">No</input>";
	}
}

function form_elt_password($option)
{
	$val=get_option($option[0]);
	print "<input id=\"$option[0]\" name=\"$option[0]\" type=\"password\" value=\"$val\" />";
}

function form_elt_string($option)
{
	$val=get_option($option[0]);
	print "<input id=\"$option[0]\" name=\"$option[0]\" type=\"text\" value=\"$val\" />";
}
								   
/* 0: option
 * 1: type
 * 2: default value
 * 3: section 
 * 4: depends on
 * 5: label
 * 6: description 
 * 7: wp version (0: all, 1:wp15 only, 2:wp2x only)
 */
function config_section($section)
{
	global $cpg_options;

	$wp_ver = function_exists('get_role') ? 2 : 1;
	
	echo '<table class="form-table" width="100%">';

	foreach($cpg_options as $option)
	{
		if ($option[3] != $section)
			continue;

		if ($option[7] && $option[7]!=$wp_ver) {
			continue;
		}

		print "<tr><td ><span class=\"help-link\" onClick=\"alert('$option[6]')\">?</span></td><td class=\"form-labels\">$option[5]:</td><td class=\"form-inputs\">";

		/* this is an ugly switch, because PHP is an ugly (but often
		 * efficient) language.
		 * There is no way to put references to functions in an array except
		 * referring to them by name and using eval. Using the PHP parser
		 * just to call a function is of course overkill and slow.  */
		
		switch($option[1]) {
			case 'user-level':
				form_elt_user_level($option);
				break;

			case 'link-style':
				form_elt_link_style($option);
				break;

			case 'charset':
				form_elt_charset($option);
				break;
			
			case 'cpg-group':
				form_elt_user_group($option);
				break;
				
			case 'int':
				form_elt_int($option);
				break;

			case 'bool':
				form_elt_bool($option);
				break;

			case 'password':
				form_elt_password($option);
				break;
			
			default:
			case 'string':
				form_elt_string($option);
				break;
		}
		print '</td></tr>';
	}

	echo '</table>';
}


function config_form()
{
	global $CPG_CONFIG;
	
print '<form method="POST">';

?>
	<div class="sectiongroup">
		<h2>Connection</h2>

		<div id="database-section" class="section">
			<h3>Database</h3>
			<?php config_section('database'); ?>
			<div class="section-buttons">
				<input type="button" onclick="open_checkdb_popup();" value="Test Connection"/>
				<br/>
			</div>
			<div class="test-result"><span id="test-result-span"></span></div>
		</div>

		<div class="section">
			<div class="section-buttons">
				<input type="submit" value="Save Settings" name="save1" /><br/>
			</div>
		</div>
	</div>

<?php
	if (isset($CPG_CONFIG['version'])) {
?>

	<div class="sectiongroup">
		<h2>Basic Settings</h2>

		<div id="options-section" class="section">
			<h3>Options</h3>
			<?php config_section('options'); ?>
		</div>

		<div id="editor-section" class="section">
			<h3>Editor Integration</h3>
			<?php config_section('editor'); ?>
<?php
			if (function_exists('get_role')) {
?>
<p>To choose which users can use the Inserter Bar, please use Owen Winkler's <a href="Role Manager">http://redalt.com/wiki/Role+Manager</a>. By default, the Administrators, Editors, Authors and Contributors can see the Inserter bar. </p>
<?php
			}
?>
				
		</div>

		<div class="section">
			<div class="section-buttons">
				<input type="submit" value="Save Settings" name="save" /><br/>
			</div>
		</div>

	</div>

	<div class="sectiongroup">
		<h2>User Integration</h2>

<?php
		if ($CPG_CONFIG['version'] < 140) {
?>

		<div id="integration-section" class="section">
		
			<p>Disclaimer: As far as I know, this plugin hasn't caused any damage to any gallery yet. But to enable user integration, it needs to modify the Coppermine database. This is not supported by the Coppermine team. So please, backup your data before you proceed.  </p>

			<p>In a many situations, user integration is not required. You can use pictures in WordPress and upload them to Coppermine without it. So only enable it if you need one of these functions:
				<ul>
					<li>You want to automatically login to Coppermine when you login to WordPress.</li>
					<li>You share your blog with other users and want them to have personal Coppermine accounts.</li>
					<li>You want registered users to be able to comment on your pictures as well as on your posts.</li>
				</ul>
			</p>

			<p>To activate user integration, two things need to be done:
				<ol>
					<li>You need to update some files in the Coppermine installation. You will have to copy the modified files from the plugin's package to your Coppermine installation manually.</li>
					<li>The Coppermine database needs to be upgraded. Use the following button to see the current status of your database. </li>
				</ol>
				<div class="section-buttons">
					<input type="button" onclick="open_checkui_popup();" value="Test Database"/>
					<br/>
				</div>
				<div class="test-result"><span id="checkdb-result-span"></span></div>
			</p>	

		</div>	

<?php
		}
?>

		<div class="section">

<?php
		if ($CPG_CONFIG['version'] < 140) {
?>

			<h3>Database Upgrade</h3>
			<p>Use this button to upgrade the database. This will alter Coppermine's user table. </p> 
			
			<div class="section-buttons">
				<input type="button" onclick="open_updatedb_popup()" value="Upgrade database"/>
				<br/>
			</div>
			<div class="test-result"><span id="updatedb-result-span"></span></div>

<?php

		}	// end if version < 140
?>
			
<?php
		if ($CPG_CONFIG['version'] < 140) {
?>

			<h4>Restore backup table</h4>
			<p>If you want to disable user integration, you can restore the users table to the state it was in before the database upgrade. This will delete all users that were imported from WordPress !</p>

			<div class="section-buttons">
				<input type="button" onclick="open_restoredb_popup()" value="Restore backup table"/>
				<br/>
			</div>
			<div class="test-result"><span id="restoredb-result-span"></span></div>

		</div>
		<div class="section">
<?php

		}	// end if version < 140
?>
			
			
			<h3>User Integration Settings</h3>
<?php
			if ($CPG_CONFIG['version'] < 140) {
?>
			<p>
				Only activate "integrate users" if the database upgrade was successful, or you will get database error messages in WordPress.
			</p>
<?php
			}
?>
			<p>
				<?php config_section('integration'); ?>
			</p>

<?php
			if (function_exists('get_role')) {
?>
<p>To choose which users share sessions with Coppermine (single login), please use Owen Winkler's <a href="Role Manager">http://redalt.com/wiki/Role+Manager</a>. By default, the Administrators, Editors, Authors and Contributors are set to auto-login. </p>
<?php
			}
?>
		<div class="section">
			<div class="section-buttons">
				<input type="submit" value="Save Settings" name="save2" /><br/>
			</div>
		</div>

				
			<h4>Import Users</h4>
			<p>Use this button to import users from WordPress into Coppermine.</p>
			
<?php if(function_exists('get_role')) { ?>
			<p>This will only import users with a user level greater or equal to Min. User level (don't forget to press "Save Settings" before you use it).</p>
<?php } else { ?>
			<p>This will import all users that have the role Administrator, Editor, Author or Contributor. (don't forget to press "Save Settings" before you use it).</p>
<?php } ?>


			<p>If you don't use this, the users will be automatically imported when they login to WordPress if "Auto-create users" is enabled.</p>
			
			<div class="section-buttons">
				<input type="button" onclick="open_import_users_popup()" value="Import WordPress Users"/>
				<br/>
			</div>
			<div class="test-result"><span id="import-users-result-span"></span></div>



		</div>

	</div>

	<div class="sectiongroup">
		<h2>Settings Management</h2>
		<div class="section">
			<div class="section-buttons">
				<input type="submit" value="Reset Settings" name="reset" /></br>
				<input type="submit" value="Uninstall" name="uninstall" />
			</div>
		</div>
	</div>

	</form>

<?php
	}	// end if version set
}






/******************************************************************************/
/*								 IMPLEMENTATION                               */
/******************************************************************************/


/*----------------------------------------------------------------------------*/
/*									SETTINGS                                  */
/*----------------------------------------------------------------------------*/

/* 0: option
 * 1: type
 * 2: default value
 * 3: section 
 * 4: depends on
 * 5: label
 * 6: description
 * 7: wp version (0: all, 1:wp15 only, 2:wp2x only)
 */

$cpg_options = array(
	/* meta */
	array( 'cpg_options_version', 	'hide',			$settings_version,	'', 			'', 					'Options set version',
		'Version of the option set for the Coppermine Integration plugin' , 0 ),
	
	/* Database stuff */
	array( 'cpg_dbserver', 			'string',		'localhost',	'database',		'',							'Server',
		'Database server for the Coppermine database' , 0 ),
	array( 'cpg_dbuser', 			'string',		'',				'database',		'',							'User',
		'Database user for the Coppermine database' , 0 ),
	array( 'cpg_dbpass',			'password',		'',				'database',		'',							'Password',
		'Database password for the Coppermine database' , 0 ),
	array( 'cpg_dbname',			'string',		'',				'database',		'',							'Database',
		'Database name for the Coppermine database' , 0 ),
	array( 'cpg_table_prefix',		'string',		'cpg133_',		'database',		'',							'Table prefix',
		'Coppermine database table prefix' , 0 ),

	/* Editor integration */
	array( 'cpg_use_inserter',		'bool',			'1',	'editor',		'',							'Use image inserter',
		'Display the one-click recent Coppermine image inserter above the editor.' , 0 ),
	array( 'cpg_inserter_num', 		'int',			'8',	'editor',		'cpg_use_inserter',			'Num. of recent images',
		'Number of images to display in the image inserter.' , 0 ),
	array( 'cpg_upload_link', 	 	'bool',			'1',	'editor',		'',							'Display upload link',
		'Place a link to upload pictures in the editor screen.' , 0 ),
	array( 'cpg_selecter_link', 	'bool',			'1',	'editor',		'',							'Display link to selecter',
		'Place a link to the Coppermine Image selecter in the editor screen.', 0 ),
	array( 'cpg_min_level',			'user-level',	'2',	'editor',		'',							'Min. User level for editor integration',
		'Minimal user-level required to see Coppermine Integration Plugin elements in the editor.', 1 ),	
	array( 'cpg_display_all_pics',	'bool',			'0',	'editor',		'',							'Display everybody\'s pictures',
		'Display all the recent pictures in the inserter bar, not only the pictures you own.', 0 ),

	/* Options */
	array( 'cpg_charset',           'charset',      'iso-8859-1', 	'options',      '',                         'Character encoding',
	    'Character encoding used in Coppermine.' , 0 ),
	array( 'cpg_link_style', 		'link-style',	'g',			'options',		'',							'Action when image is clicked',
		'Select what happens when you click on an image.', 0 ),
	array( 'cpg_image_style', 		'bool',	'0',			'options',		'',							'Use div-tag for displaying image? (no=img-tag)',
		'How the image will be displayed, streched img or cropped div (all images will have the same size and orientation)', 0 ),

	array( 'cpg_div_size_x',		'int',		'100',				'options',		'',	 						'div width',
		'div width' , 0 ),
	array( 'cpg_div_size_y',		'int',		'75',				'options',		'',	 						'div height',
		'div height' , 0 ),
	array( 'cpg_div_options',		'string',		'',				'options',		'',	 						'background extra',
		'extra options which will be added to background style tag' , 0 ),
	array( 'cpg_show_filename', 		'bool',	'1',			'options',		'',							'Display filename',
		'Show filename / description below image', 0 ),



	array( 'cpg_popup_upload', 		'bool',			'1',			'options',		'cpg_upload_link',			'Use popup upload window',
		'Display a popup window to upload pictures when the upload link is clicked. wp-upload.php needs to be present in the Coppermine directory.' , 0 ),
	array( 'cpg_upload_user_id',		'int',		'',				'options',		'',	 						'Coppermine user id',
		'Userid for just selecting the own pictures. (empty for all)' , 0 ),
	array( 'cpg_upload_user',		'string',		'',				'options',		'',	 						'Coppermine user',
		'Username for automatic login, not used if user integration if enabled.' , 0 ),
	array( 'cpg_upload_pass',		'password',		'',				'options',		'',							'Coppermine password',
		'Password for automatic login, not used if user integration if enabled.' , 0 ),
	array( 'cpg_use_dewplayer',     'bool',         '1',            'options',      '',                         'Use DewPlayer for Mp3',
	    'Use Dew\'s lightweight player (flash) for Mp3.' , 0 ),
	array( 'cpg_use_shadows',       'bool',         '0',            'options',      '',                         'Display a shadow around pictures',
		'Display a shadow around pictures.' , 0 ),

	/* User Integration */
	array( 'cpg_integrate_users', 	'bool',			'0',	'integration',		'',						'Integrate users',
		'Enable/Disable integration of Wordpress and Coppermine users. Does not work with Coppermine 1.4', 0 ),
	array( 'cpg_auto_create_users',	'bool',			'1',	'integration',		'cpg_integrate_users',	'Auto-create users',
		'Automatically create a Coppermine user for every WordPress user at login.' , 0 ),
	array( 'cpg_auto_create_album',	'bool',			'1',	'integration',		'cpg_integrate_users',	'Auto-create albums',
		'Automatically create a default album for every WordPress user at login. ', 0 ),
	array( 'cpg_user_group',       	'cpg-group',	'2',    'integration',		'cpg_integrate_users',	'Coppermine group',
		'Coppermine group WordPress users should be added to. ', 0 ),
	array( 'cpg_user_min_level_create', 'user-level','2',	'integration',		'cpg_integrate_users',	'Min. User level',
		'Minimum user-level required to get a Coppermine login.' , 1 ),
	array( 'cpg_auto_login',		'bool',			'1',	'integration',		'cpg_integrate_users',	'Automatically login',
		'Automatically login to Coppermine when you login to WordPress.' , 0 ),
	array( 'cpg_remote_login',		'bool',			'1',	'integration',		'cpg_auto_login',		'Remote login',
		'You need this option if you want to automatically login to a Coppermine server located on a remote server. ' , 1 ),
);





/*----------------------------------------------------------------------------*/
/*									  HTML                                    */
/*----------------------------------------------------------------------------*/

function html_head($status)
{
?>
<html>
  <head>
    <title>Coppermine integration plugin for Wordpress - Configuration</title>

	<style type="text/css" media="screen">

body {
	text-align: left;
	padding: 0;
	margin: 0;
	font-family: Georgia, Verdana, Arial;
}

#body {
	margin: 0 auto;
	padding: 15px;
	width: 600px;
	border-left: solid black 2px;
	border-right: solid black 2px;
	background-color: #eaeaea;
}

#status {
	color: red;
	padding-bottom: 2em;
	text-align: center;
}


.form-table {
	position: relative;
}

.help-link {
	background-color: #ddf;
	color: red;
	padding: 0 5px;
	border: solid #666 1px;
	cursor: help;
}

.help-link:hover {
	background-color: #eef;
}

.form-labels {
	width: 240px;
	padding: 3px 0;
}

.form-inputs {
	padding: 0 30px;
}

.sectiongroup {
	background-color: #f2f2f2;
	border: dashed 1px #333;
	padding: 1em;
	margin-bottom: 2em;
	padding-bottom: 4em;
}

.section {
	margin-top: 40px;
}

.section p {
	padding-bottom: 10px;
}

.section-buttons {
	margin-top: 20px;
	text-align: center;
}

.section-buttons input {
	padding: 2px 20px;
	margin-bottom: 10px;
	width: 200px;
}

.section-buttons input:hover {
	border-color: #333;
	color: red;
	background-color: #eef;
}

h1 {
	text-align: center;
	background-color: white;
	padding: 10px 0;
	margin: 20px 0 0;
	border-top: solid #333 1px;
	border-bottom: solid #333 1px;
	font-family: Georgia, Verdana, Arial;
	letter-spacing: 0.3em;
}

#linkbar {
	text-align: center;
	width: 600px;
	margin: 0 auto 20px;
	font-size: 80%;
}

#linkbar a, #linkbar a:visited {
	text-decoration: none;
	color: #66d;
}

#linkbar a:hover {
	text-decoration: underline;
}

h2 {
	text-align: center;
	font-family: Georgia, Verdana, Arial;
	letter-spacing: 0.1em;
}

h3 {
	font-family: Georgia, Verdana, Arial;
	letter-spacing: 0.1em;
}

.test-result {
	text-align: left;
	margin: 10px auto;
}

#test-result-span {
	text-align: left;
}

	</style>

	<script language="javascript" type="text/javascript">
function mkuri(action)
{
	var server=escape(document.getElementById('cpg_dbserver').value);
	var user=escape(document.getElementById('cpg_dbuser').value);
	var pass=escape(document.getElementById('cpg_dbpass').value);
	var db=escape(document.getElementById('cpg_dbname').value);
	var prefix=escape(document.getElementById('cpg_table_prefix').value);
	return '<?php echo $_SERVER['PHP_SELF']; ?>?action='+action+'&s='+server+'&u='+user+'&p='+pass+'&db='+db+'&px='+prefix;
}
	
function open_checkdb_popup() {
	window.open(mkuri('checkdb'),'checkdb','scrollbars=no,status=no,width=10,height=10');
}

function open_checkui_popup() {
	window.open(mkuri('checkui'),'checkui','scrollbars=no,status=no,width=10,height=10');
}

function open_updatedb_popup() {
	window.open(mkuri('updatedb'),'updatedb','scrollbars=no,status=no,width=10,height=10');
}

function open_restoredb_popup() {
	window.open(mkuri('restoredb'),'restoredb','scrollbars=no,status=no,width=10,height=10');
}

function open_import_users_popup() {
	window.open(mkuri('import-users'), 'import-users', 'scrollbars=no,status=no,width=10,height=10');
}

	
	</script>
	
  </head>
  <body>
  <div id="body">
  <h1>Coppermine Settings</h1>
  <div id="linkbar">
  <a href="../../../wp-admin/plugins.php">Plugins Administration</a> - <a href="../../../">View Site</a> - <a href="http://www.stilglog.com/wordpress-plugins/coppermine-plugin/">Online Documentation</a>
  </div>
  <p><small>Make sure Javascript is enabled and popup windows are allowed.</small></p>
  <div id="status"><?php echo $status; ?></div>
<?php
}

function html_food()
{
?>
	<div class="section">
		<p>This plugin is copyright Martijn van der Kwast 2005. </p>
		<p>More information is available <a href="http://www.stilglog.com/wordpress-plugins/coppermine-plugin/">here</a>.</p>
	</div>


	</div>
  </body>
<html>
<?php
}



/*----------------------------------------------------------------------------*/
/*								 POPUP WINDOWS                                */
/*----------------------------------------------------------------------------*/

function insert_write_span_js( $id, $msg, $close )
{
	global $CPG_CONFIG;

	$spans = array( 'test-result-span', 'checkdb-result-span', 
					'updatedb-result-span', 'restoredb-result-span', 
					'import-users-result-span' );

	$msg = str_replace("'",'&apos;',$msg);
?>
	<script language="javascript" type="text/javascript">
		var obj=null;
<?php	foreach( $spans as $span ) { ?>
			obj=opener.document.getElementById('<?php echo $span; ?>');
			if (obj) obj.innerHTML = '';
<?php	} ?>
	
		var span=opener.document.getElementById('<?php echo $id; ?>');	
		span.innerHTML = '<?php echo $msg; ?>';
<?php if ($close) { ?>
		window.close();
<?php } ?>
	</script>
<?php
}

/* test database connection */
function window_checkdb_connection()
{
	$r = _connection_test();
	$retval = $r[0];
	$retmsg = $r[1];

	insert_write_span_js( 'test-result-span', $retmsg, true );
}


/* Test if system is ready for user integration */
function window_check_ui()
{
	$r = _connection_test();
	$retval = $r[0];
	$retmsg = $r[1];

	global $cpg_cmdb;

	if (!$retval) {
		insert_write_span_js( 'checkdb-result-span', $retmsg, true );
		return;
	}

	$r = _user_integration_test();
	$retval = $r[0];
	$retmsg = $r[1];

	if (!$retval) {
		insert_write_span_js( 'checkdb-result-span', $retmsg, true );
		return;
	}

	if (!_wordpress_ok()) {
		insert_write_span_js( 'checkdb-result-span', 'There is a problem with the database connections. Disable user-integration, and please report this.', true );
		return;
	}

	insert_write_span_js( 'checkdb-result-span', $retmsg, true );
}

/* update database */
function window_updatedb()
{
	global $CPG_CONFIG;

	/* don't do anything if Coppermine is modern */
	if ($CPG_CONFIG['version']>=140) {
		return;
	}
	
	$r = _connection_test();
	$retval = $r[0];
	$retmsg = $r[1];

	/* check db connection */
	if (!$retval) {
		insert_write_span_js( 'updatedb-result-span', $retmsg, true );
		return;
	}

	/* check if already patched */
	$r = _can_patch();
	if (!$r) {
		insert_write_span_js( 'updatedb-result-span', 'The database is up to date and does not need an upgrade.', true );
		return;
	}

	/* backup + upgrade */
	$r = alter_table();
	$retval = $r[0];
	$retmsg = $r[1];

	// alter failed
	if (!$retval) {
		insert_write_span_js( 'updatedb-result-span', $retmsg, true );
		return;
	}

	// double check if update was ok
	$r = _user_integration_succeeded();
	$retval = $r[0];
	$retmsg = $r[1];
	if (!$retval) {
		insert_write_span_js( 'updatedb-result-span', $retmsg, true );
		return;
	}

	// connection to wordpress is borked
	if (!_wordpress_ok()) {
		insert_write_span_js( 'checkdb-result-span', 'There is a problem with the database connections. Disable user-integration, and please report this.', true );
		return;
	}

	insert_write_span_js( 'updatedb-result-span', $retmsg, true );
}

/* restore database backup */
function window_restoredb()
{
	$r = _connection_test();
	$retval = $r[0];
	$retmsg = $r[1];

	if (!$retval) {
		insert_write_span_js( 'restoredb-result-span', $retmsg, true );
		return;
	}
	
	$r = restore_backup();
	$retval = $r[0];
	$retmsg = $r[1];
	
	insert_write_span_js( 'restoredb-result-span', $retmsg, true );
}

/* import WordPress users into CPG */
function window_import_users()
{
	global $wpdb;
	global $CPG_CONFIG;
	global $table_prefix;
	
	/* check db connection */
	$r = _connection_test();
	$retval = $r[0];
	$retmsg = $r[1];

	if (!$retval) {
		insert_write_span_js( 'import-users-result-span', $retmsg, true );
		return;
	}

	/* check if database has been updated */
	$r = _user_integration_test();
	$retval = $r[0];
	$retmsg = $r[1];
	if (!$retval) {
		insert_write_span_js( 'import-users-result-span', $retmsg, true );
		return;
	}

	/* get all WordPress users with level > integration level */
	if ($CPG_CONFIG['version'] < 140) {
		$min_level=intval(get_option('cpg_user_min_level_create'));
		$q = "SELECT * FROM $wpdb->users WHERE user_level > $min_level";
		$wpdb->query($q);
		$users = $wpdb->get_results();
	} else {
		/* In Wp2.0+ we import all active users with roles != subscriber */
		$q = "SELECT * FROM {$wpdb->users} u, {$wpdb->usermeta} m where u.ID = m.user_id and meta_key='".$table_prefix."capabilities'";
		
		$wpdb->query($q);
		$users_tmp = $wpdb->get_results();

		$users = array();
		foreach($users_tmp as $tmp) {
			$caps = unserialize($tmp->meta_value);
			foreach($caps as $cap => $s) {
				fwrite($f,"$cap: $s\n");
			}
			if (!$caps['inactive'] && ($caps['contributor'] || $caps['administrator'] || $caps['editor'] || $caps['author'])) {
				array_push($users, $tmp);
			}
		}
	}

	$i=0;
	//$msg = '';
	if ($users) foreach ($users as $user) {
		if (cpg_import_user($user)) {
			++$i;
			$msg .= $user->user_login.', ';
		}
	}

	if ($i) {
		$msg = substr($msg, 0, -2);
		$msg = "$i users were imported into the Coppermine user list: ".$msg;
	} else {
		$msg = 'No users were imported into the Coppermine user list.';
	}

	if ($CPG_CONFIG['version'] < 140) {
		$msg="Import of users with level > $min_level: <br/>".$msg;
	} else {
		$msg="Import of users: <br/>".$msg;
	}
	insert_write_span_js( 'import-users-result-span', $msg, true);
}


/*----------------------------------------------------------------------------*/
/*									 TESTS                                    */
/*----------------------------------------------------------------------------*/

// return (true, msg) if can connect
function _connection_test()
{
	require_once('./coppermine.php');

	global $wpdb;
	global $cpg_cmdb;
	global $CPG_CONFIG;

	print "<h1>Testing connection...</h1>";

	$server = $_GET['s'];
	$user = $_GET['u'];
	$pass = $_GET['p'];
	$db = $_GET['db'];
	$prefix = $_GET['px'];

	$retval = true;
	$retmsg = '';
	
	if (DB_HOST == $server &&
	    DB_USER == $user &&
		DB_PASSWORD == $pass &&
		DB_NAME == $db )
	{
		$cpg_cmdb = new cpg_wpdb_clone($wpdb);
		$retmsg = ' The database connection is shared with WordPress.';
	}
	else {
		$cpg_cmdb = @new cpg_wpdb($server,$user,$pass,$db);
		$retmsg = ' The database connection is not shared with WordPress.';
	}

	if (!$cpg_cmdb || !$cpg_cmdb->dbh) {
		$retval = false;
		$retmsg = 'Can not connect to server or login incorrect. ';
	} else {
		// read cpg CONFIG table
		$CPG_CONFIG=array('TABLE_CONFIG' => $prefix.'config');
		$sql="SELECT name, value FROM {$CPG_CONFIG['TABLE_CONFIG']}";
		@$cpg_cmdb->query($sql);
		$results=@$cpg_cmdb->get_results();

		if (count($results)) {
			$retmsg = 'Connection successful. <br/>'.$retmsg;
		} else {
			$retmsg = 'The connection to the database was successful, but the Coppermine tables could not be found. This probably means that the table prefix you entered is invalid. It is usually something like &quot;<code>cpg133_</code>&quot; or &quot;<code>cpg140_</code>&quot;.<br/>'.$retmsg;
			$retval = false;
		}
	}

	// detect version
	if ($retval) {
		$sestab=get_option('cpg_table_prefix').'sessions';
		$sql="SHOW TABLES LIKE '".$sestab."'";
		if ($sestab==$cpg_cmdb->get_var($sql)) {
			$CPG_CONFIG['version'] = 140;
		} else {
			$CPG_CONFIG['version'] = 130;
		}
	}

	return array($retval, $retmsg);
}

// return (true, msg) if patched 
function _user_integration_test()
{
	global $cpg_cmdb;
	global $CPG_CONFIG;
	

	$q='SELECT * FROM '.$_GET['px'].'users LIMIT 1';
	@$r=$cpg_cmdb->get_row($q, ARRAY_A);
	if (!$r) {
		$retval = false;
		$retmsg = 'Your Coppermine database seems to be corrupt: the users table is not accessible. '."$q";
	} 
	else {
		if ($CPG_CONFIG['version'] < 140) {
			// cpg 1.3
			if (isset($r['user_password_md5'])) {
				$retval = true;
				$retmsg = 'The Coppermine database is ready for user integration and does not need an update. Make sure you have also updated the files in your Coppermine installation.';
			}
			else {
				$retval = false;
				$retmsg = 'The Coppermine database needs to be updated. Use the "Upgrade database" link to make it ready for user integration.';
			}
		} else {
			// cpg 1.4
			$retval = true;
			$retmsg = 'The Coppermine database is ready for user integration and does not need an update. Make sure you have also updated the files in your Coppermine installation.';
		}
	}

	return array($retval, $retmsg);
}

// return (true, msg) if patched 
function _user_integration_succeeded()
{
	global $cpg_cmdb;
	
	$q='SELECT * FROM '.$_GET['px'].'users LIMIT 1';
	@$r=$cpg_cmdb->get_row($q, ARRAY_A);
	if (!$r) {
		$retval = false;
		$retmsg = 'Your Coppermine database seems to be corrupt: the users table is not accessible. '."$q".format_sqlerror();
	} 
	else {
		if ($CPG_CONFIG['version'] < 140) {
			// cpg 1.3
			if (isset($r['user_password_md5'])) {
				$retval = true;
				$retmsg = 'Database upgraded and backup created.';
			}
			else {
				$retval = false;
				$retmsg = 'An error occured while updating the database.'.format_sqlerror();
			}
		} else {
			// cpg 1.4
			$retval = true;
			$retmsg = 'Database ready.';
		}
	}

	return array($retval, $retmsg);
}



// return (true, msg) if can upgrade
function _can_patch()
{
	global $cpg_cmdb;
	
	$q='SELECT * FROM '.$_GET['px'].'users LIMIT 1';
	$r=@$cpg_cmdb->get_row($q, ARRAY_A);
	if (!$r) {
		return false;
	} 
	else {
		if (isset($r['user_password_md5'])) {
			return false;
		}
		else {
			return true;
		}
	}
}

// return true if wordpress still works...
function _wordpress_ok()
{
	global $wpdb;

	$q="SELECT * FROM $wpdb->options LIMIT 1";
	return ($wpdb->query($q)>0);
}



/*----------------------------------------------------------------------------*/
/*								DATABASE ACTIONS                              */
/*----------------------------------------------------------------------------*/

function query_failed()
{
	global $EZSQL_ERROR;
	$err=$EZSQL_ERROR[1]; //mysql_error();
	return (!empty($err));
}

function format_sqlerror()
{
	global $EZSQL_ERROR;
	$retmsg='';
	$err=mysql_error();
	if (empty($err)) {
		$err=$EZSQL_ERROR['error_str'];
	}
	if (!empty($err)) {
		$retmsg.='<br/>(mysql: '.htmlspecialchars($err).')';
	}
	return $retmsg;
}


// safe backup: first rename, only drop when everything is in place
function restore_backup()
{
	global $cpg_cmdb;

	$q='SELECT * FROM '.$_GET['px'].'users_backup LIMIT 1';
	if (!$cpg_cmdb->query($q) or query_failed()) {
		return array(false, 'No backup found.'.format_sqlerror());
	}

	$q='RENAME TABLE '.$_GET['px'].'users TO '.$_GET['px'].'users_temp';
	if (false===$cpg_cmdb->query($q) or query_failed()) {
		return array(false, 'Could not move the modified table away.'.format_sqlerror());
	}

	$q='RENAME TABLE '.$_GET['px'].'users_backup TO '.$_GET['px'].'users';
	if (false===$cpg_cmdb->query($q) or query_failed()) {
		return array(false, 'Could not restore the table from the backup.'.format_sqlerror());
	}

	$q='DROP TABLE '.$_GET['px'].'users_temp';
	if (false===$cpg_cmdb->query($q) or query_failed()) {
		return array(false, 'Could not remove the modified table.'.format_sqlerror());
	}

	return array(true, 'Table restored from backup, and backup removed. You will also need to restore the original files in the Coppermine directory !');
}

// return (true, msg) on success
function alter_table()
{
	global $cpg_cmdb;
	global $CPG_CONFIG;

	if ($CPG_CONFIG['version']>=140) {
		return array(false, 'This version of coppermine does not need an upgrade');
	}

	$q='CREATE TABLE '.$_GET['px'].'users_backup AS SELECT * FROM '.$_GET['px'].'users';
	if (false===$cpg_cmdb->query($q) or query_failed()) {
		return array(false, 'Could not backup the table '.$_GET['px'].'users, do you have sufficient permissions ? '.format_sqlerror());
	}

	$q='ALTER TABLE '.$_GET['px'].'users ADD COLUMN ( user_password_md5 VARCHAR(64) )';
	if (false===$cpg_cmdb->query($q) or query_failed()) {
		return array(false, "Could not modify the ".$GET['px']."users table. Do you have sufficient permissions ?".format_sqlerror() );
	}

	$q='UPDATE '.$_GET['px'].'users SET user_password_md5 = MD5( user_password )';
	if (false===$cpg_cmdb->query($q) or query_failed()) {
		return array(false, "Could not encode the passwords. You should probably restore the table from the backup.", format_sqlerror() );
	}

	return array(true, 'Database upgraded and backup created.');
}

function check_login()
{
	global $user_ID;

	if (!$user_ID) {
		get_currentuserinfo();
	}

	if (!$user_ID) {
	$_GET['redirect_to'] = '%2Fwp-content%2Fplugins%2Fcoppermine%2F';
wp_redirect('../../../wp-login.php');

#		html_head('<h3>Not logged in.</h3>');
#		html_food();
		exit;
	}

	if (function_exists('get_role')) {
		// wp 2.0+
		if (!current_user_can('activate_plugins')) {
wp_redirect('../../../wp-login.php');
/*			html_head('<h3>Not logged in.</h3>
			<h3>You need to be at least a level 8 trampling+flying poweruser</h3>
			<small>Come back when you have more experience points.</small>');
			html_food();
			*/
			exit;
		}
	} else {
		// wp 1.5
		$user=get_userdata($user_ID);
		if ($user->user_level < 8 ) {
wp_redirect('../../../wp-login.php');
/*			html_head('<h3>Not logged in.</h3>
			<h3>You need to be at least a level 8 trampling+flying poweruser</h3>
			<small>Come back when you have more experience points.</small>');
			html_food();*/
			exit;
		}
	}

}

/*----------------------------------------------------------------------------*/
/*									 ROLES                                    */
/*----------------------------------------------------------------------------*/

function update_roles()
{
	// wp 2.0+ only
	if (!function_exists('get_role')) {
		return;
	}

	// don't set default caps if the admin
	// already has cpg caps set.
	$role = get_role('administrator');
	if ($role) {
		if ($role->has_cap('cpg_auto-login') ||
			$role->has_cap('cpg_display_all_pics'))
		{
			return;
		}
	}
	
	// set caps for admin
	if ($role) {
		$role->add_cap('cpg_auto-login');
		$role->add_cap('cpg_display_inserter');
	}

	// set caps for contributor
	$role = get_role('contributor');
	if ($role) {
		$role->add_cap('cpg_auto-login');
		$role->add_cap('cpg_display_inserter');
	}

	// set caps for editor
	$role = get_role('editor');
	if ($role) {
		$role->add_cap('cpg_auto-login');
		$role->add_cap('cpg_display_inserter');
	}

	// set caps for authors
	$role = get_role('author');
	if ($role) {
		$role->add_cap('cpg_auto-login');
		$role->add_cap('cpg_display_inserter');
	}
}


/*----------------------------------------------------------------------------*/
/*									  MAIN                                    */
/*----------------------------------------------------------------------------*/


require_once('../../../wp-blog-header.php');
require_once('./coppermine.php');

check_login();
update_roles();

// dispatcher for popup windows
switch ($_GET['action']) {
	case 'checkdb':
		window_checkdb_connection();
		exit;

	case 'checkui':
		window_check_ui();
		exit;

	case 'updatedb':
		window_updatedb();
		exit;

	case 'restoredb':
		window_restoredb();
		exit;

	case 'import-users':
		window_import_users();
		exit;
}

// handle actions
if (isset($_POST['uninstall'])) {
	settings_uninstall();
	print <<<UNINST
<html><head><title>Settings removed</title></head><body>
<h1>Settings removed from database. Adios !</h1>
</body><html>
UNINST;
	exit;
}

if (isset($_POST['save']) || 
	isset($_POST['save1']) ||
	isset($_POST['save2'])) {
	settings_save();
	$status = 'Settings saved.';
}

if (isset($_POST['reset'])) {
	settings_reset();
	$status = 'Settings reset to defaults.';
}

if (!settings_are_installed()) {
	$status = 'Default settings created.';
	settings_install();
}

if ($settings_version != get_option('cpg_options_version')) {
	$status = 'Settings updated.';
	settings_update();
	update_option('cpg_options_version', $settings_version);
}

@cpg_connect();

// display main page
html_head($status);
config_form();
html_food();
?>
