<?php
/*
Plugin Name: Coppermine
Version: 4.0
Plugin URI: http://blog.artworkz.net/?p=283
Description: Integrate Coppermine into Wordpress. Go <a href="../wp-content/plugins/coppermine/coppermine-config.php">here</a> to configure.
Author: started: Martijn van der Kwast, Matthias Jell, Jason Goldsmith; continued by Mathias Decker
Author URI: http://blog.artworkz.net

Copyright 2005 Martijn van der Kwast (email : mvdkwast@gmx.met)

Please use the coppermine-config script to change the options,
and don't modify this file.

See the end of the file for licencing.

Written for Wordpress version 1.5 and Coppermine version 1.33-1.35
Changed to work with Wordpress 2.xx

Information about how to use/install can be found here:
http://www.stilglog.com/wordpress-plugins/coppermine-plugin/

*/
/////////////////////////////////////////////////////////////////////





/******************************************************************************/
/*								    CONSTANTS                                 */
/******************************************************************************/



/* the following settings should not be modified for normal use. They can be  */
/* subject to wild changes between versions.                                  */
/* The important stuff has been moved to WP options, and should be configured */
/* through coppermine-config.php .                                            */






/* If this is enabled and the database is unreachable,
   this plugin will act as if it is disabled instead of displaying
   an error screen. The error will be a comment in the HTML source.
   (Use View > Source in your browser).
*/
$CPG_CONCEAL_ERRORS = true;

/* text for album link if there is no title */
define ('CPG_ALBUM_TEXT', 	   'visit this album');  

/* fallback group for privileges if the configured group is bogus */
define ('CPG_GUESTGROUP',      3 );

/* allowed codes in tags */
$cpg_style_codes = array(
	't'	=> 'title',
	'l' => 'link',
	's' => 'shadow',
	'b' => 'border',
	'bw' => 'border-width',
	'bh' => 'border-height',
	'c' => 'class',
	'w' => 'width',
	'h' => 'height',
	'i' => 'condition'
);

/* Ok, this can be useful actually.  Define custom styles by modifying this array. 
 * This defines how the :y part in [image:xx:y] is expanded in tags.
 */
$cpg_alignments=array(
	'l' => array(
		'Left', 
		'<div style="text-align:center; float: left">', 
		'</div>'),
	'r' => array(
		'Right', 
		'<div style="text-align:center; float: right">', 
		'</div>'),
	'c' => array(
		'Center', 
		'<div style="text-align:center; display: block; margin: auto auto;">',
		'</div>'),
	'i' => array(
		'Inline',
		'', ''),
);


/* defaults for non-registered users in cpg */
$cpg_default_privs = array(
	'can_rate_pictures'		=> false,
	'can_send_ecards'		=> false,
	'can_post_comments'    	=> false,
	'can_upload_pictures'	=> false,
	'can_create_albums'     => false,
	'has_admin_access'      => false,  /* this also means can_see_all_albums */
	'pub_upl_need_approval' => true,
	'priv_upl_need_approval' => false
);
		
/* end Obscure options */

/******************************************************************************/
/*					   YOU SHOULDN'T HAVE TO CHANGE BELOW                     */
/******************************************************************************/


// globals


/*
 * Coppermine configuration array
 * Equivalent to $CONFIG in the CPG sources
 */
$CPG_CONFIG = array();

/*
 * Database connection to Coppermine
 */
$cpg_cmdb = null;


/*
 * CPG base URL, extracted from $CPG_CONFIG
 */
$cpg_base = '';

/*
 * CPG base url for pictures, normally $cpg_base/albums
 */
$cpg_uri = '';

/*
 * Filetypes, required to handle extensions other than pictures
 * This is loaded the first time it is used and used as a cache to save 
 * some db requests.
 */
$cpg_filetypes = null;


/*
 * Small icons for documents that are not pictures
 * wp-root/wp-content/plugins/coppermine/img is assumed
 */
$cpg_filetype_icons = array(
	'default'	=> 'document-txt.png',
	'movie' 	=> 'tv.png',
	'audio' 	=> 'musicnote.png',
	'document' 	=> 'document-txt.png'
);

/*
 * location of the DewPlayer
 */
$cpg_dewplayer = '/wp-content/plugins/coppermine/dewplayer/dewplayer.swf';


/******************************************************************************/
/*							  THE FILTER FUNCTION                             */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/*								LAYOUT FUNCTIONS                              */
/*----------------------------------------------------------------------------*/

/* 
 * Manipulate style hash (inefficient semantisms)
 */
function cpg_default_style(&$style, $key, $value) {
	if (!isset($style[$key])) {
		$style[$key]=$value;
	}
}

function cpg_override_style(&$style, $key, $value) {
	$style[$key]=$value;
}

function cpg_default_style_set(&$style, &$override) {
	foreach($override as $opt => $val) {
		if (!isset($style[$opt])) {
			$style[$opt] = $val;
		}
	}
}

function cpg_override_style_set(&$style, &$override) {
	foreach($override as $opt => $val) {
		$style[$opt] = $val;
	}
}

 


/*
 * wrapping function for a picture. specifies what happens when
 * you click on a picture.
 */
function _cpg_link($html_img, $image, $link_style)
{
	global $cpg_link_styles;
	global $cpg_base;
	
	// if no explicit link style is set for non image types, 
	// link directly to the resource instead of to the album or 
	// whereever
	if (!$link_style || empty($link_style)) {
		if (cpg_is_image($image)) {
			$link_style=get_option('cpg_link_style');
		} else {
			$link_style='d';
		}
	}

	if (empty($link_style)) {
		// cannot find any linkstyle
		return $html_img;
	}

	if (isset($cpg_link_styles[$link_style])) {
		$opt = $cpg_link_styles[$link_style];

		if ($opt) {
			// get formatting function returning the content
			// of the href attribute from table.
			// return the original string if 
			// - there is no matching function
			// - the return value is an empty string

			$fhref=$opt[1];
			if (empty($fhref)) {
				return $html_img;
			}

			$href=$fhref($image);
			if (empty($href)) {
				return $html_img;
			}
			return "<a style=\"background: transparent;\" class=\"cpg-link\" rel=\"lightbox[my]\" href=\"$href\">$html_img</a>";
		} else {
			// a code specifying no link
			return $html_img;
		}
	} else {
		// an url
		return "<a class=\"cpg-link\" href=\"$link_style\">$html_img</a>";
	}

}

/*
 * close and open paragraph around div
 */ 
function _cpg_fmt_p($str)
{
	return '</p>'.$str.'<p>';
}

/*
 * helper function for the filter. 
 * inserts a picture in a div to align it.
 */
function _cpg_fmt($str,$fmt)
{
	global $cpg_alignments;

	$alignment=$cpg_alignments[$fmt];
	if (isset($alignment)) {
		$str = $alignment[1].$str.$alignment[2];
	} else {
		$str = preg_replace('|</a><br/>.*|','</a>',$str);
	}
	return $str;
}

/*
 * Put shadow around picture
 */
function _cpg_shadow($html)
{
	return <<<HTML
<table style="display: inline;" class="shadow" border="0" 
    cellspacing="0" cellpadding="0">  
  <tr>
    <td class="shadow-topLeft"></td>
    <td class="shadow-top"></td>
    <td class="shadow-topRight"></td>
   </tr>
   <tr>
     <td class="shadow-left"></td>
     <td class="shadow-center">
       $html
     </td>
     <td class="shadow-right"></td>   
   </tr>
   <tr>
     <td class="shadow-bottomLeft"></td>
     <td class="shadow-bottom"></td>
     <td class="shadow-bottomRight"></td>
   </tr>
</table>    
HTML;

}

/*----------------------------------------------------------------------------*/
/*								 TAG EXPANSION                                */
/*----------------------------------------------------------------------------*/


/*
 * Decode a tag style argument string in the form:
 * - :         -> implicit alignment
 * - :l        -> explicit alignment
 * - ::a=val   -> implicit alignment and options+value
 * - :l:a=val  -> explicit alignment and options+value
 *
 * the values can contain ':' as long as they don't contain ':X='
 *
 * all values present in $override will not be set
 */
function cpg_push_arg(&$args, $key, $val) {
	if (!empty($key)) { // && !empty($val)) {
		$av = explode('=',$val,2); 
		$args[$key]=$av[1];
	}
}

function _cpg_explode_args($str) {
	global $cpg_style_codes;
	$args = array();

	if ($str[0] != ':') {
		return array();
	}

	// Decode alignment and remove first field
	$matches = array();
	if (!preg_match('|^:[^:]+=|', $str, $matches)) {
		preg_match( '/^:(\w+)/', $str, $matches);
		$align = $matches[1];
		$str = preg_replace( '/^:\w+/', '', $str);
		$args['align']=$align;
	}
	
	// process next arguments
	$bits = explode(':', $str);

	$nbits = count($bits);
	$matches = array();
	$lastkey = '';
	$lastarg = '';
	if ($nbits < 2) {
		return $args;
	}

	for($i=1; $i<$nbits; ++$i) {
		$bit=$bits[$i];

		if (preg_match('/^(\w)=/',$bit,$matches)) {
			cpg_push_arg($args, $lastkey, $lastarg);
			$lastkey = $cpg_style_codes[$matches[1]];
			//if (!isset($cpg_style_codes[$matches[1]])) {
			//	print "Warning: unkown key in descriptor: '$matches[1]'. Contact your local Microsoft retailer.\n";
			//}
			$lastarg=$bit;
		} else {
			$lastarg.=':'.$bit;
		}
	}

	cpg_push_arg($args, $lastkey, $lastarg);
	return $args;
}

/*
 * arguments for groups and args for images in groups are separated
 * by '::'
 */
function _cpg_explode_group_args($txt)
{
	if (preg_match('/(.*?):(:.*)/',$txt,$matches)) {
		return array( 
			_cpg_explode_args($matches[1]),
			_cpg_explode_args($matches[2])
		);
	} else {
		return array(
			_cpg_explode_args($txt),
			array()
		);
	}
}
 

/*
 * Main filter function
 * Used to transform [image] tags into <div ..><a ..><img blabal /></a></div>
 */
function cpg_filter($text) {

	cpg_connect();
	
	$old_len = strlen($text);
	
   	// using a hash would have been nicer, preg_replace has its limitations.
	$fmt='(:[^\]]*)?';

	$text = preg_replace(
		array(
			"/\[group\]((?:.*\n)*?.*?)\[\/group\]/me",
			"/\[group$fmt\]((?:.*\n)*?.*?)\[\/group\]/me"
		),
		array(
			'_cpg_fmt_p(_cpg_group(\'\',\'$1\'))',
			'_cpg_fmt_p(_cpg_group(\'$1\',\'$2\'))',
		),
		$text
	);
    
	$text = preg_replace( 
		array(
			"/\[album:(\d+)$fmt\]/e",
			"/\[album:(\d+),(\d+)$fmt\]/e",
			"/\[image:(\d+)$fmt\]/e",
	    	"/\[image:(\d+),(\d+)$fmt\]/e",
		   	"/\[thumb:(\d+)$fmt\]/e",
		   	"/\[thumb:(\d+),(\d+)$fmt\]/e",
			"/\[newline\]/e",
		),
		array(
			'_cpg_fmt_p(cpg_get_album_wrap($1,0,\'$2\'))', 
			'_cpg_fmt_p(cpg_get_album_wrap($1,$2,\'$3\'))',
			'_cpg_fmt_p(cpg_get_image_absolute($1,_cpg_explode_args(\'$2\')))',
			'_cpg_fmt_p(cpg_get_image($1, $2,_cpg_explode_args(\'$3\')))',
			'_cpg_fmt_p(cpg_get_thumbnail_absolute($1,_cpg_explode_args(\'$2\')))',
			'_cpg_fmt_p(cpg_get_thumbnail($1, $2,_cpg_explode_args(\'$3\')))',
			'_cpg_fmt_p(cpg_get_newline())',
		),
		$text);

	if (strlen($text) != $old_len) {

		// add clearing element
		$text = $text . '<div class="clearer">&nbsp;</div>';
	}
		
	return $text;
}

/* remove <p></p>'s */
function cpg_de_p_er($text) {
	return str_replace('<p></p>','',$text);
}


/*
 * link styles, choose what happens when you click on a picture, sorry for the ns pollution,
 * but i'm **not** going to use create_function to put functions in a hash.
 */
function _cpg_link_href_nada($image) {
	return '';
}

function _cpg_link_href_gallery($image) {
	global $cpg_base;
	return "{$cpg_base}displayimage.php?pos=-{$image->pid}";
}

function _cpg_link_href_popup($image) {
	global $cpg_base;
	$width=$image->pwidth+20;
	$height=$image->pheight+20;
	if (cpg_is_image($image)) {
		return "javascript:void(window.open('{$cpg_base}displayimage.php?pid={$image->pid}&amp;fullsize=1','','toolbar=no,status=no,resizable=yes,width={$width},height={$height}'))";
	} else {
		$uri=cpg_urlencode(_cpg_get_image_uri($image));
		return "javascript:void(window.open('$uri'));";
	}
}

function _cpg_link_href_direct($image) {
	return cpg_urlencode(_cpg_get_image_uri($image));
}


$cpg_link_styles=array(
	'g' => array(
		'View in Gallery',
		'_cpg_link_href_gallery' ),
	'x' => array(
		'Do nothing',
		'_cpg_link_href_nada'),
	'p' => array(
		'Open in popup',
		"_cpg_link_href_popup"),
	'd' => array(
		'Direct link',
		'_cpg_link_href_direct'),
);


/*
 * process a group tag
 */
function _cpg_group($rrstyle, $content)
{
	$images = array();
	
	$styles = _cpg_explode_group_args($rrstyle);
	$style = $styles[0];
	$istyle = $styles[1];

	// find images in group
	$fmt='(:[^\]]*)?';

	$n=preg_match_all( 
		"/\[(image|thumb):(?:(\d+,\d+|\d+))$fmt\]/",
		$content,
		$tags
	);

	// no matches no work, don't bother with a border
	if ($n==0) {
		return '';
	}

	// construct images array: get images from db and 
	// parse options
	for($i=0;$i<$n;$i++) {
		if (preg_match('/(\d+),(\d+)/', $tags[2][$i], $pids)) {
			$image=cpg_db_get_image($pid[1], $pid[2]);
		} else {
			$image=cpg_db_get_imagedata($tags[2][$i]);
		}

		$options = _cpg_explode_args($tags[3][$i]);
		$options['thumb']= ( $tags[1][$i]=='image' ? false : true );

		cpg_default_style_set($options, $istyle);

		array_push($images, array($image, $options));
	}

	// Use cpg_format_group to render the images and group
	return cpg_format_group($images, $style);
}

 
/******************************************************************************/
/*						 ENCODING CONVERSION FUNCTIONS                        */
/******************************************************************************/

/**
 * Encodes UTF-8 characters to HTML entities
 *
 * @author <vpribish at shopping dot com>
 * @link   http://www.php.net/manual/en/function.utf8-decode.php
 */
if (!function_exists(utf8_tohtml)) {
	 
	function utf8_tohtml ($str) {
	  $ret = '';
	  $max = strlen($str);
	  $last = 0;  // keeps the index of the last regular character
	  for ($i=0; $i<$max; $i++) {
		$c = $str{$i};
		$c1 = ord($c);
		if ($c1>>5 == 6) {  // 110x xxxx, 110 prefix for 2 bytes unicode
		  $ret .= substr($str, $last, $i-$last); // append all the regular characters we've passed
		  $c1 &= 31; // remove the 3 bit two bytes prefix
		  $c2 = ord($str{++$i}); // the next byte
		  $c2 &= 63;  // remove the 2 bit trailing byte prefix
		  $c2 |= (($c1 & 3) << 6); // last 2 bits of c1 become first 2 of c2
		  $c1 >>= 2; // c1 shifts 2 to the right
		  $ret .= '&#' . ($c1 * 100 + $c2) . ';'; // this is the fastest string concatenation
		  $last = $i+1;
		}
	  }
	  return $ret . substr($str, $last, $i); // append the last batch of regular characters
	} 

}

/*
 * Convert CPG encoding to WP
 * FIXME: comparing everytime is NOT efficient, but it
 * should still be fast compared to the SQL ;) 
 */
function cpg_recode(&$str)
{
	if (empty($str)) {
		return '';
	}
	
	$cpg_charset = get_option('cpg_charset');
	$blog_charset = get_option('blog_charset');
	
	$out = '';
	
	if (function_exists('mb_convert_encoding')) { 
		$out = @mb_convert_encoding($str, $blog_charset, strtoupper($cpg_charset));
	} 
	
	if (empty($out) && function_exists('recode_string')) {
		$out = @recode_string($cpg_charset, $blog_charset, $str);
	}
		
	if (empty($out) && function_exists('iconv')) {
		$out = @iconv($cpg_charset, $blog_charset, $str);
	}

	if (empty($out) && function_exists('libiconv')) {
		$out = @libiconv($cpg_charset, $blog_charset, $str);
	} 
	
	if (empty($out) && function_exists('utf8_encode')) {
		// only supports latin1, but should still produce valid utf8 for 
		// any string, even if it is gibberish
		$out = @utf8_encode($str);
	}

	$str = utf8_tohtml($out);
	return $str;
}
 

/******************************************************************************/
/*							   DATABASE FUNCTIONS                             */
/******************************************************************************/


/*----------------------------------------------------------------------------*/
/*							  WRAP WP's wpdb CLASS                            */
/*----------------------------------------------------------------------------*/


/*
 * WP's database class is reused here because it's nicely done, and because it
 * will make WP hackers more comfortable with this code. Some functions however
 * need to be overriden because we want to die with an error message like WP
 * does if we can't connect to the CPG DB. This way we can still read posts and
 * change plugin settings.  The links to CPG will be hidden if the DB is
 * offline. This way we won't scare off visitors every time the CPG DB is
 * offline. (but only when the main DB is offline q; )
 */ 


/*
 * Override the some wpdb methods.  
 * if CPG_CONCEAL_ERRORS is true: don't let anyone know, except for the missing
 * pictures and a comment.  
 * if CPG_CONCEAL_ERRORS is false: abort after error.
 */
class cpg_wpdb extends wpdb {

	/*
	 * Constructor
	 */
	function cpg_wpdb($server,$user,$pass,$dbname) { 
		
		// the true in connect is _very_ important, without WP's DB connection is replaced.
		$this->dbh = @mysql_connect($server, $user, $pass, true);

		if (!$this->dbh) {
			$this->bail("<h2>Error establishing a connection to the <em>Coppermine</em> database.</h2>
<p>Most probably the database is offline, or the settings are incorrect.</p>");
		}
		else {
			$this->select($dbname);
		}
	}

	/*
	 * Database selection, replaced because WP's error message is a bit too
	 * long for a secondary DB error.
	 */
	function select($db) {
		if (!@mysql_select_db($db, $this->dbh)) {
			$this->bail("<h2>Error selecting the <em>Coppermine</em> database</h2>
<p>We were able to connect to the database server, but not able to select the <code>$db</code> database.</p>");
			$this->dbh=false;
		}
	}

	/*
	 * Abort with error, or hide error in HTML source if $CPG_CONCEAL_ERRORS enabled
	 */
	function bail($message) {
		global $CPG_CONCEAL_ERRORS;
		if ($CPG_CONCEAL_ERRORS) print '<!-- ';
		print $message;
		if ($CPG_CONCEAL_ERRORS) print '--> ';
		$this->print_error($str);
		if (!$CPG_CONCEAL_ERRORS) {
			die();
		}
	}

	/*
	 * Display error message.
	 */
	function print_error($str = '') {
		global $CPG_CONCEAL_ERRORS;
		global $EZSQL_ERROR;
		if (!$str) $str = mysql_error();
		$EZSQL_ERROR[] = 
		array ('query' => $this->last_query, 'error_str' => $str);

		// Is error output turned on or not..
		if ( $this->show_errors ) {
			// If there is an error then take note of it
			if ($CPG_CONCEAL_ERRORS) print '<!-- ';

			print "<div id='error'>
			<p class='wpdberror'><strong>Coppermine database error:</strong> [$str]<br />
			<code>$this->last_query</code></p>
			</div>";

			if ($CPG_CONCEAL_ERRORS) print '--> ';
		}
	}
}

/*
 * Wrapper class for cpg_wpdb, to reuse WP's DB connection.
 * Since PHP doesn't support function overloading, we can't overload the constructor.
 * But since all methods are virtual, we can just inherit the class instead and
 * create a new constructor.
 */
class cpg_wpdb_clone extends cpg_wpdb {
	function cpg_wpdb_clone($wpdb) {
		$this->dbh = $wpdb->dbh;
	}
}

/*
 * Stub class replacing wpdb
 * We use an instance of this instead of setting cpg_cmdb to false so that we
 * don't get errors if a functions tries to access the database when it's
 * offline. While I agree this can be seen as bad practice, it's _useful_
 * practice. Every function could check if the database if online before
 * proceeding, but if you miss one in a rarely used function you would get an
 * ugly error message. With this solution, you don't.
 */
class cpg_wpdb_stub {
	function cpg_wpdb_stub() {
		$this->unavailable=true;
	}
	function select($db) {}
	function escape($str) {}
	function print_error($str='') {}
	function show_errors() {}
	function hide_errors() {}
	function flush() {}
	function query($query) {}
	function get_var($query=null, $x = 0, $y = 0) {}
	function get_row($query = null, $output = OBJECT, $y = 0) {}
	function get_col($query = null , $x = 0) {}
	function get_results($query = null, $output = OBJECT) {}
	function get_col_info($info_type = 'name', $col_offset = -1) {}
	function timer_start() {}
	function timer_stop($precision = 3) {}
	function bail($message) {}
}



/*----------------------------------------------------------------------------*/
/*								   CONNECTION                                 */
/*----------------------------------------------------------------------------*/

function cpg_fix_uri_add_slash($str) {
	$l=strlen($str)-1;
	if ($l>=0) {
		return ($str[$l]!='/') ? $str.'/' : $str;
	} else {
		return '';
	}
}

/*
 * connect to database, try to reuse connection
 * initialize $cpg_uri and $cpg_base
 */
function cpg_connect()
{
	global $CPG_CONFIG;
	global $cpg_cmdb;
	global $cpg_base;
	global $cpg_uri;
	global $wpdb;

	// don't connect twice
	if (isset($cpg_cmdb)) {
		return;
	}

	// connect: recycle WP connection if CPG
	// and WP share the same DB.
	if (DB_HOST == get_option('cpg_dbserver') &&
	    DB_USER == get_option('cpg_dbuser') &&
		DB_PASSWORD == get_option('cpg_dbpass') &&
		DB_NAME == get_option('cpg_dbname' ))
	{
		$cpg_cmdb = new cpg_wpdb_clone($wpdb);
	}
	else {
		$cpg_cmdb = new cpg_wpdb(
			get_option('cpg_dbserver'), 
			get_option('cpg_dbuser'), 
			get_option('cpg_dbpass'), 
			get_option('cpg_dbname')
		);
	}

	if (!$cpg_cmdb || !$cpg_cmdb->dbh) {
		$cpg_cmdb = new cpg_wpdb_stub();
		return;
	}

	// read cpg CONFIG table
	$CPG_CONFIG=array('TABLE_CONFIG' => get_option('cpg_table_prefix').'config');
	$sql="SELECT name, value FROM {$CPG_CONFIG['TABLE_CONFIG']}";
	$cpg_cmdb->query($sql);

	$results=$cpg_cmdb->get_results();
	foreach($results as $v) {
		$CPG_CONFIG[$v->name]=$v->value;
	}

	$cpg_base = cpg_fix_uri_add_slash($CPG_CONFIG['ecards_more_pic_target']);
	$cpg_uri = $cpg_base.$CPG_CONFIG['fullpath'];


	// detect Coppermine 1.4 (FIXME: we waste a select here, version should be in options)
	$sestab=get_option('cpg_table_prefix').'sessions';
	$sql="SHOW TABLES LIKE '".$sestab."'";
	if ($sestab==$cpg_cmdb->get_var($sql)) {
		$CPG_CONFIG['version'] = 140;
	} else {
		$CPG_CONFIG['version'] = 130;
	}
}

/*----------------------------------------------------------------------------*/
/*						  GENERAL INFO FROM CPG TABLES                        */
/*----------------------------------------------------------------------------*/


/*
 * Get all info about a user
 */
function cpg_db_get_userdata($uid)
{
	global $cpg_cmdb;

	$sql= 'SELECT * '.
		  'FROM '.get_option('cpg_table_prefix').'users '.
		  'WHERE user_id='.intval($uid);
	$cpg_cmdb->query($sql);
	$groups=$cpg_cmdb->get_row();

	return $groups;
}


/*
 * Get all info about a group
 */
function cpg_db_get_groupdata($guid)
{
	global $cpg_cmdb;

	$sql= 'select * '.
		'from '.get_option('cpg_table_prefix').'usergroups '.
		'where group_id='.intval($guid);
	$cpg_cmdb->query($sql);
	return $cpg_cmdb->get_row();
}

/*
 * Get all groups for a users. Called _db_ because every
 * sensible human being would thing this comes from a joining 
 * db table, not a varchar.
 */
function cpg_db_get_user_groups($uid)
{
	global $cpg_cmdb;
	$user = cpg_db_get_userdata($uid);
	if (!$user) {
		return array();
	}
	
	$groups=array();
	array_push($groups, $user->user_group);
	if (!in_array($user->user_group, $groups)) {
		array_push($groups, $user->user_group);
	}

	return $groups;
}
 

/*
 * Get all info about an album
 */
function cpg_db_get_albumdata($aid)
{
	global $cpg_cmdb;
		
	$sql= 'SELECT * '.
		  'FROM '.get_option('cpg_table_prefix').'albums '.
		  'WHERE aid='.$aid;
	$cpg_cmdb->query($sql);
	$album = $cpg_cmdb->get_row();
	cpg_recode($album->title);
	cpg_recode($album->description);
	cpg_recode($album->keyword);
	return $album;
}


/*
 * Get ids from all active users
 */
function cpg_db_get_users()
{
	global $cpg_cmdb;

	$sql='SELECT user_id FROM '.get_option('cpg_table_prefix').'users WHERE user_active=\'YES\'';
	$cpg_cmdb->query($sql);
	return $cpg_cmdb->get_col();
}

/*
 * get all user groups
 */
function cpg_db_get_groups()
{
	global $cpg_cmdb;

	$sql= 'select * '.
		'from '.get_option('cpg_table_prefix').'usergroups ';
	$cpg_cmdb->query($sql);
	return $cpg_cmdb->get_results();
}

/*
 * get userdata for all users. (This could be a lot of data for a big userbase).
 */
function cpg_db_get_all_userdata()
{
	global $cpg_cmdb;
	
	$sql='SELECT * FROM '.get_option('cpg_table_prefix').'users WHERE user_active=\'YES\'';
	$cpg_cmdb->query($sql);
	return $cpg_cmdb->get_results();
}

/*
 * get image row
 */
function cpg_db_get_imagedata($pid) 
{
	global $cpg_cmdb;

	$cpg_cmdb->query(
		'SELECT * '.
		'FROM '.get_option('cpg_table_prefix').'pictures '.
		"WHERE pid=$pid ");
	$image = $cpg_cmdb->get_row();
	cpg_recode($image->title);
	cpg_recode($image->caption);
	cpg_recode($image->keywords);
	return $image;
}

/* retrieve comments for an image */
function cpg_db_get_image_comments($pid, $num=0)
{
	global $cpg_cmdb;

	$q = 'SELECT * FROM '.get_option('cpg_table_prefix').'comments '.
	     'WHERE pid = '.intval($pid).' '.
		 'ORDER BY msg_date DESC ';
	if ($num > 0) {
		$q .= 'LIMIT '.$num;
	}

	$comments = $cpg_cmdb->get_results($pid);
	if ($comments) foreach($comments as $comment) {
		$comment->msg_author = cpg_recode($comment->msg_author);
		$comment->msg_body = cpg_recode($comment->msg_body);
	}

	return $comments;
}

function cpg_db_get_recent_comments($num=0)
{
	global $cpg_cmdb;

	$q = 'SELECT * FROM '.get_option('cpg_table_prefix').'comments '.
	     'ORDER BY msg_date DESC';
	if ($num > 0) {
		$q .= 'LIMIT '.$num;
	}

	$comments = $cpg_cmdb->get_results($pid);
	if ($comments) foreach($comments as $comment) {
		$comment->msg_author = cpg_recode($comment->msg_author);
		$comment->msg_body = cpg_recode($comment->msg_body);
	}

	return $comments;
}


/*----------------------------------------------------------------------------*/
/*		   					  RETRIEVE PICTURES                               */
/*----------------------------------------------------------------------------*/


/*
 * get images in album
 */
function cpg_db_get_album_images($aid,$num=0,$random=false) 
{
	global $cpg_cmdb;
		
	// get album thumbnail paths and titles
	$sql=
		'SELECT * '.
		'FROM '.get_option('cpg_table_prefix').'pictures '.
		'WHERE aid='.$aid.' ';

	if ($random) {
		$sql.=' ORDER BY RAND() ';
	} else {
		# FIXME: order by something from config
		$sql.= 'ORDER BY pid ';
	}

	if ($num) {
		$sql.=' LIMIT '.$num;
	}

	$cpg_cmdb->query($sql);
	$images = $cpg_cmdb->get_results();

	if ($images) {
		for ($i=0; $i<count($images); ++$i) {
			cpg_recode($images[$i]->title);
			cpg_recode($images[$i]->caption);
			cpg_recode($images[$i]->keywords);
		}
	}
	
	return $images;
}


/*
 * get image defined by aid and pos
 */
function cpg_db_get_image($aid, $pos) 
{
	global $cpg_cmdb;
	
	// get picture path and title
	$cpg_cmdb->query(
		'SELECT * '.
		'FROM '.get_option('cpg_table_prefix').'pictures '.
		"WHERE aid=$aid ".
		"LIMIT 1 OFFSET $pos" );
	$image = $cpg_cmdb->get_row();
	cpg_recode($image->title);
	cpg_recode($image->caption);
	cpg_recode($image->keywords);
	return $image;
}

/*
 * get last images
 */
function cpg_db_get_last_images($num, $user_id=0)
{
	global $cpg_cmdb;

	$sql=
	'SELECT pictures.* '.
    'FROM '.get_option('cpg_table_prefix').'pictures AS pictures, '.
    '     '.get_option('cpg_table_prefix').'albums AS album '.
	'WHERE pictures.aid = album.aid AND ';

	if($user_id && !get_option('cpg_display_all_pics')) {
		$sql.=' pictures.owner_id='.intval($user_id).' ';
	}
	else {
		$sql.=' album.visibility=0 ';
	}
	$sql.=
	'ORDER BY pid DESC ' .
	'LIMIT '.$num;

    $cpg_cmdb->query($sql);
	$images = $cpg_cmdb->get_results();
	if ($images) {
		for ($i=0; $i<count($images); $i++) {
			cpg_recode($images[$i]->title);
			cpg_recode($images[$i]->caption);
			cpg_recode($images[$i]->keywords);
		}
	}

	return $images;
}

/*
 * get random images
 */
function cpg_db_get_random_images($num, $user_id=0)
{
	global $cpg_cmdb;

	$sql=
	'SELECT pictures.* '.
    'FROM '.get_option('cpg_table_prefix').'pictures AS pictures, '.
	'     '.get_option('cpg_table_prefix').'albums AS albums '.
	'WHERE pictures.aid = albums.aid AND ';

	if($user_id) {
		$sql.=' pictures.owner_id='.(0+$user_id).' ';
	} else {
		$sql.=' albums.visibility=0 ';
	}
	$sql.=
	'ORDER BY RAND() ' .         // FIXME: Performance issue 
	'LIMIT '.$num;

	$cpg_cmdb->query($sql);
	$images = $cpg_cmdb->get_results();
	if ($images) {
		for ($i=0; $i<count($images); $i++) {
			cpg_recode($images[$i]->title);
			cpg_recode($images[$i]->caption);
			cpg_recode($images[$i]->keywords);
		}
	}
	
	return $images;
}

/*
 * get albums for CPG user
 */
function cpg_db_get_albums($user_id=0)
{
	
	global $cpg_cmdb;

	$sql=
	'SELECT * '.
	'FROM '.get_option('cpg_table_prefix').'albums ';

	if ($user_id>1) {
		$sql .= 'WHERE category = '.(10000+intval($user_id)).' ';
	} else if ($user_id==1) {
		$sql .= 'WHERE category <= 10001 ';
	} else {
		$sql .= 'WHERE visibility=0 ';
		/* else: all albums */
	}

	$sql .= 'ORDER BY title ASC';
	
	$cpg_cmdb->query($sql);
	$albums = $cpg_cmdb->get_results();
	
	if ($albums) {
		for ($i=0; $i<count($albums); ++$i) {
			cpg_recode($albums[$i]->title);
			cpg_recode($albums[$i]->description);
			cpg_recode($albums[$i]->keyword);
		}
	}
	return $albums;
}

/*
 * get album thumbnail
 */
function cpg_db_get_album_thumbnail($aid)
{
	global $cpg_cmdb;

	// get official thumb
	$sql = 'SELECT thumb FROM '.get_option('cpg_table_prefix').'albums WHERE aid='.$aid;
	$thumb = $cpg_cmdb->get_var($sql);

	if (!$thumb) {
		// default is last uploaded
		$sql = 'SELECT max(pid) FROM '.get_option('cpg_table_prefix').'pictures WHERE aid='.$aid;
		$thumb = $cpg_cmdb->get_var($sql);
	}

	return $thumb;
}

/*----------------------------------------------------------------------------*/
/*								USER MANAGEMENT                               */
/*----------------------------------------------------------------------------*/


/*
 * Update last visit
 */
function cpg_touch_user($user_id) 
{
	global $cpg_cmdb;

	$cpg_cmdb->query(
		"UPDATE ".get_option('cpg_table_prefix')."users " . 
		"SET user_lastvisit = NOW() " .
		"WHERE user_id = '" . $cpg_cmdb->escape($user_id) . "' AND " .
		"user_active = 'YES'");
}

/*
 * return user_id, user_name and user_password for cpg user
 */
function cpg_get_login_info($user_login) 
{
	global $cpg_cmdb;
	global $CPG_CONFIG;

	if (!$cpg_cmdb) {
		return null;
	}
	
	if (intval(get_option('cpg_integrate_users'))) {
		if ($CPG_CONFIG['version'] >= 140) {
			# FIXME: handle encrypted passwords
			$sql = "SELECT user_id, user_name, user_password " .
				"FROM " . get_option('cpg_table_prefix').'users ' .
				"WHERE user_name = '" . $cpg_cmdb->escape($user_login) . "' AND " .
				"user_password IS NOT NULL AND " .
				"user_active = 'YES' ";
		} else {
			$sql = "SELECT user_id, user_name, user_password_md5 " .
				"FROM " . get_option('cpg_table_prefix').'users ' .
				"WHERE user_name = '" . $cpg_cmdb->escape($user_login) . "' AND " .
				"user_password_md5 IS NOT NULL AND " .
				"user_active = 'YES' ";
		}
	} else {
		$sql = "SELECT user_id, user_name, md5(user_password) " .
			"FROM " . get_option('cpg_table_prefix').'users ' .
			"WHERE user_name = '" . $cpg_cmdb->escape($user_login) . "' AND " .
			"user_password IS NOT NULL AND " .
			"user_active = 'YES' ";
	}

	$cpg_cmdb->query($sql);
	$user = $cpg_cmdb->get_row();
	return $user;
}

/*
 * get a user list
 */
function cpg_get_users()
{
	global $cpg_cmdb;

	$sql = "SELECT * " .
		"FROM " . get_option('cpg_table_prefix').'users ' .
		"user_active = 'YES' ";
	$cpg_cmdb->query($sql);
	$users = $cpg_cmdb->get_results();
	return $users;
}
 
/*
 * Import a user from wordpress to coppermine
 */
function cpg_import_user($wp_user)
{
	global $cpg_cmdb;
	global $CPG_CONFIG;

	if ($CPG_CONFIG['version'] < 140) {
		// cpg 1.3
		$sql = 
		'INSERT INTO '.get_option('cpg_table_prefix').'users ( ' .
		'  user_group, user_active, user_name, user_password, ' .
		'  user_lastvisit, user_regdate, user_email, user_website, '.
		'  user_password_md5 ) ' .
		'VALUES ( '.
		   intval(get_option('cpg_user_group')) . ', ' .
		   "'YES', " .
		   "'{$wp_user->user_login}', " .
		   rand().', ' .
		   "now()," .
		   "now()," .
		   "'{$wp_user->user_email}', " .
		   "'{$wp_user->user_url}', " .
		   "'{$wp_user->user_pass}' ) ";

		return @$cpg_cmdb->query( $sql );
	} else {
		// cpg 1.4

		// although constraints should disallow duplicate entries, some
		// ppl seem to have them disabled. So let's choose for the safe
		// path, and not import users that already have an equivalent in
		// the coppermine user database
		$sql =
		'SELECT COUNT(*) FROM '.get_option('cpg_table_prefix').'users '.
		"WHERE user_name = '".$wp_user->user_login."'";
		if ($cpg_cmdb->get_var($sql) > 0) {
			return null;
		}
		
		// FIXME: setting the password will only work if password
		// encryption is enabled in coppermine
		$sql = 
		'INSERT INTO '.get_option('cpg_table_prefix').'users ( ' .
		'  user_group, user_active, user_name, user_password, ' .
		'  user_lastvisit, user_regdate, user_email) ' .
		'VALUES ( '.
		   intval(get_option('cpg_user_group')) . ', ' .
		   "'YES', " .
		   "'{$wp_user->user_login}', " .
		   "'{$wp_user->user_pass}', " .
		   "now()," .
		   "now()," .
		   "'{$wp_user->user_email}' ) ";

		return @$cpg_cmdb->query( $sql );
	}
}

/*
 * Create a default album for the new user
 */
function cpg_create_default_album($cpg_user)
{
	global $cpg_cmdb;

	$sql=
	'SELECT aid FROM '.get_option('cpg_table_prefix').'albums WHERE category = '.($cpg_user->user_id+10000);

	if (0 == $cpg_cmdb->get_var($sql)) {
		$sql=
		'INSERT INTO '.get_option('cpg_table_prefix').'albums ( title, category )' .
		"VALUES ( 'Default', ". (10000+$cpg_user->user_id) . ')';
		$cpg_cmdb->query($sql);
	}
}

/*
 * Get privileges for user, straight from CPG init.inc.php
 * The fields in the return value that can be used are the same as in 
 * $cpg_default_privs.
 */
function cpg_db_get_user_privileges($uid)
{
	global $CPG_CONFIG;
	global $cpg_cmdb;
	global $cpg_default_privs;
	
	if (!$uid) {
		return $cpg_default_privs;
	}
	
	$user = cpg_db_get_userdata($uid);
	if (!$user) {
		return $cpg_default_privs;
	}
	
	$groups = cpg_db_get_user_groups($uid);

	if (empty($groups)) {
		$groups=array(CPG_GUESTGROUP);
	}
	
	/* only left what could be useful in WP */
	$sql = "SELECT ". 
           "  MAX(can_rate_pictures) as can_rate_pictures, " .
		   "  MAX(can_send_ecards) as can_send_ecards, " .
           "  MAX(can_post_comments) as can_post_comments, ".
		   "  MAX(can_upload_pictures) as can_upload_pictures, ".
           "  MAX(can_create_albums) as can_create_albums, " .
           "  MAX(has_admin_access) as has_admin_access, " .
           "  MIN(pub_upl_need_approval) as pub_upl_need_approval, ".
		   "  MIN(priv_upl_need_approval) as  priv_upl_need_approval ".
           "FROM ".get_option('cpg_table_prefix')."usergroups " .
		   "WHERE group_id in (" .  implode(',', $groups). ")";

   $cpg_cmdb->query($sql);
   $privs = $cpg_cmdb->get_row();

   // get privs for default group no results
   if (!$privs) {
	   $sql = "SELECT * FROM ".get_option('cpg_table_prefix')."usergroups WHERE group_id=".CPG_GUESTGROUP;
	   $cpg_cmdb->query($sql);
	   $privs = $cpg_cmdb->get_row();
   }

   return $privs;

}

/*
 * Check if a user can acces an album
 */
function cpg_can_access_album($cpguserid, $aid)
{
	global $CPG_CONFIG;
	
	if (!intval(get_option('cpg_integrate_users'))) {
		return true;
	}
	
	if (!($CPG_CONFIG['allow_private_albums'])) {
		return true;
	}
	
	$album = cpg_db_get_albumdata($aid);
	if ($album->visibility == '0' || 
	    $album->visibility == 10000 + $cpguserid) 
	{
		return true;
	}

	$groups = cpg_db_get_user_groups($cpguserid);
	if (in_array($album->visibility,$groups)) {
		return true;
	}

	$privs = cpg_db_get_user_privileges($cpguserid);
	if ($privs->has_admin_access) {
		return true;
	}
	
	return false;
}



/*----------------------------------------------------------------------------*/
/*								SEARCH FUNCTIONS                              */
/*----------------------------------------------------------------------------*/

/*
 * Return a list of aids matching the keywords array
 * FIXME: this will be slow for big databases, calculate accessible 
 *        albums first instead.
 */

/**/
function cpg_db_search_albums($keywords)
{
	global $CPG_CONFIG;
	global $cpg_cmdb;
	global $user_ID;

	$wp_user = get_userdata($user_ID);
	$cpg_user = cpg_get_login_info($wp_user->user_login);

	$keywords=$cpg_cmdb->escape($keywords);
	$keywords=preg_split('/\s+/',$keywords);
	if (!count($keywords)) {
		return array();
	}

	$phrase='';
	foreach ($keywords as $key) {
		$phrase.="OR title LIKE '%$key%' OR description LIKE '%key%' ";
	}
	$phrase=preg_replace('/^OR /','',$phrase);

	$q = 'SELECT aid FROM '.get_option('cpg_table_prefix').'albums WHERE '.$phrase;

	$cpg_cmdb->query($q);
	$albums=$cpg_cmdb->get_results();

	$ret = array();
	if ($albums) foreach ($albums as $album) {
		if (cpg_can_access_album($cpg_user->user_id, $album->aid)) {
			array_push($ret, $album->aid);
		}
	}
	return $ret;
}

/**/

/**/

/* Return a list of pids matching the keywords array
 * FIXME: this will be _very_ slow. 
 *        cache the permissions + calculate accessible albums first to save time
 */

function cpg_db_search_pictures($keywords)
{
	global $CPG_CONFIG;
	global $cpg_cmdb;
	global $user_ID;

	$wp_user = get_userdata($user_ID);
	$cpg_user = cpg_get_login_info($wp_user->user_login);

	$phrase=$cpg_cmdb->escape($keywords);

	$q = 'SELECT pid,aid FROM '.get_option('cpg_table_prefix').'pictures WHERE '.
	     "MATCH ( filename, title, caption, keywords ) AGAINST ( '$phrase' ) ".
		 'AND approved = \'YES\'';

	$cpg_cmdb->query($q);
	$pictures=$cpg_cmdb->get_results();
	$ret=array();

	if ($pictures) foreach($pictures as $pic) {
		if (cpg_can_access_album($cpg_user->user_id, $pic->aid)) {
			array_push($ret, $pic->pid);
		}
	}

	return $ret;
}


/**/



/******************************************************************************/
/*								 UTILITY FUNCTIONS                            */
/******************************************************************************/

/* encode uri's, but leave '/',':' */
function cpg_urlencode($path) {
	return str_replace (array('%2F','%3A','%25'),array('/',':','%'), rawurlencode ($path));	
}

/* get image uri, if normal_ does not exist, return full sized
 * this is done by checking cp config settings */
function _cpg_get_image_uri($image)
{
	global $CPG_CONFIG;
	global $cpg_uri;

	// The weird comparison is because only picture_width is stored
	if ($CPG_CONFIG['thumb_use']=='ht' && $image->pheight > $CPG_CONFIG['picture_width']) { 
		$usenormal = true;
	} elseif($CPG_CONFIG['thumb_use']=='wd' && $image->pwidth > $CPG_CONFIG['picture_width']) {
		$usenormal = true;
	} elseif($CPG_CONFIG['thumb_use']=='any' && max($image->pwidth, $image->pheight) > $CPG_CONFIG['picture_width']) {
		$usenormal = true;
    } else{
		$usenormal = false;
    }

    if ($CPG_CONFIG['make_intermediate'] && $usenormal ) {
		$uri = $cpg_uri.$image->filepath.'normal_'.$image->filename;
    } else {
		$uri = $cpg_uri.$image->filepath.$image->filename;
    }

	return cpg_urlencode($uri);
}

/* get thumbnail uri */
function _cpg_get_thumb_uri($image)
{
	global $cpg_uri;
	
	if (cpg_is_image($image)) {
		$uri = cpg_urlencode($cpg_uri.$image->filepath.'thumb_'.$image->filename);
	} else {
		$uri = _cpg_get_other_thumbnail($image);
	}
	return $uri;
}


/* Insert link to stylesheet */
function _cpg_insert_stylesheet()
{
?>
<link rel="stylesheet" href="<?php echo get_settings('siteurl'); ?>/wp-content/plugins/coppermine/coppermine.css" type="text/css" media="screen" /><?php
}

/* Insert javascript for image viewers */
function _cpg_insert_javascript()
{
?>
	<script type="text/javascript" src="<?php echo get_settings('siteurl'); ?>/wp-content/plugins/coppermine/slide_show2.js" ></script>
<?php
}


/*
 * Return a link to a thumbnail for a non-image file.
 * Of course in Coppermine's 3v1l naming even a .mp3 file will be 
 * an image...
 */
function _cpg_get_other_thumbnail($image)
{
	global $cpg_base;
	$parts=explode('.',$image->filename);
	$ext=strtolower($parts[count($parts)-1]);
	$uri=cpg_urlencode($cpg_base.'images/thumb_'.$ext.'.jpg');
	return $uri;
}


/******************************************************************************/
/*								   MULTIMEDIA                                 */
/******************************************************************************/

function _cpg_load_filetypes()
{
	global $cpg_cmdb;
	global $cpg_filetypes;

	if ($cpg_filetypes == null) {
		$sql = 'SELECT extension, content FROM '.get_option('cpg_table_prefix').'filetypes';
		$results=$cpg_cmdb->get_results($sql);

		$cpg_filetypes = array();
		foreach($results as $result) {
			$cpg_filetypes[$result->extension] = $result->content;
		}
	}
}

/*
 * return content type for extension
 */
function cpg_get_filetype($image)
{
	global $cpg_filetypes;

	_cpg_load_filetypes();
	
	$parts=explode('.',$image->filename);
	$ext=strtolower($parts[count($parts)-1]);

	if (!isset($cpg_filetypes[$ext])) {
		return null;
	}
	$content = $cpg_filetypes[$ext];
	return $content;
}

function cpg_is_image($image)
{
    return ('image'==cpg_get_filetype($image));
}

function cpg_is_movie($image)
{
    return ('movie'==cpg_get_filetype($image));
}

function cpg_is_audio($image)
{
    return ('audio'==cpg_get_filetype($image));
}

function cpg_is_document($image)
{
    return ('document'==cpg_get_filetype($image));
}

function cpg_is_known_imagetype($image)
{
    return cpg_is_image($image) || cpg_is_movie($image) || cpg_is_audio($image) || cpg_is_document($image);
}

function cpg_is_mp3($image)
{
	$parts=explode('.',$image->filename);
	$ext=strtolower($parts[count($parts)-1]);
	return ($ext=='mp3');
}

function cpg_get_filetype_icon($image)
{
	global $cpg_filetype_icons;

	$parts=explode('.',$image->filename);
	$ext=strtolower($parts[count($parts)-1]);
	$content=cpg_get_filetype($image);

	if (isset($cpg_filetype_icons[$content])) {
		$icon=$cpg_filetype_icons[$content];
	}
	else {
		$icon=$cpg_filetype_icons['default'];
	}

	return get_option('siteurl').'/wp-content/plugins/coppermine/img/'.$icon;
}


/******************************************************************************/
/*								 TEMPLATE TAGS                                */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/*						 LAYOUT MEAT AND SOUP AND SALT                        */
/*----------------------------------------------------------------------------*/

// options: 
// thumb:    bool
// align:    l,r,c,(none)     if not set, no enclosing block will be generated
// link:     g,x,p,(link)     if not set, use cpg_link_style
// title:    (title)          if not set, a default title will be used
// class:    (css class)
// shadow:   bool			  if not set, use cpg_use_shadow

function cpg_format_image($image, $options)
{
	global $cpg_dewplayer;

	// disable linking if a media player is inserted
	$inhibit_link=false;

	// disable shadows if it's not a picture
	$inhibit_shadow=false;

	// title style
	$title_style='';   // pre, inline
	
	if (isset($options['title'])) {
		$title=$options['title'];
	}
	else {
		$title = $image->title;
		if (empty($title)) {
			$title=$image->filename;
		}
	}

	// inline image style is completely naked
	if ($options['align']=='i') {
		$options['bare']=true;
	}

	// create class="blah" statement
	if (isset($options['class'])) {
		$class=' class="'.$options['class'].'"';
	} else {
		$class='';
	}

	// handle extra img options (width, height)
	$extra_html;
	if (isset($options['width'])) {
		$extra_html.=' width="'.intval($options['width']).'"';
	}
	if (isset($options['height'])) {
		$extra_html.=' height="'.intval($options['height']).'"';
	}

	// generate image tag
	$is_image = cpg_is_image($image);
	if (!$is_image && ($options['in_group'] || !$options['thumb'])) {
		$inhibit_shadow=true;

		$uri = _cpg_get_image_uri($image);

		// handle non-image files
		if (cpg_is_mp3($image) && get_option('cpg_use_dewplayer')) {
			$inhibit_link=true;
			$title_style='pre';
			$dewplayer = get_option('siteurl').$cpg_dewplayer;
			$html=<<<DEW
	<object type="application/x-shockwave-flash" width="200" height="20" data="$dewplayer?son=$uri"> 
	</object>
DEW;
		} else {
			$title_style='inline';
//				$uri=cpg_get_filetype_icon($image);
			$uri = _cpg_get_thumb_uri($image);
			$html="<img src=\"$uri\" style=\"height: 16px;\" alt=\"$title\" title=\"$title\" $class/>";
		}
	}
	else if ($options['thumb']) {
		// thumbnails
		$uri = _cpg_get_thumb_uri($image);
		if (!cpg_is_image($image)) {
			$inhibit_shadow=true;
		}
		if(get_option('cpg_image_style')==0) {
		    $html="<img src=\"$uri\" alt=\"$title\" title=\"$title\" {$class}{$extra_html}/>";
		} else {
		    $lens="<img src=\"./wp-content/plugins/coppermine/img/empty.gif\" alt=\"$title\" title=\"$title\" style=\"width: ".(get_option('cpg_div_size_x')-20)."px; height: ".(get_option('cpg_div_size_y')-20)."px; cursor: hand;\"/><img src=\"./wp-content/plugins/coppermine/img/empty.gif\" alt=\"$title\" title=\"$title\" style=\"width: 20px; height: ".(get_option('cpg_div_size_y')-20)."px; cursor: hand;\"/><br>";
		    $lens.="<img src=\"./wp-content/plugins/coppermine/img/empty.gif\" alt=\"$title\" title=\"$title\" style=\"width: ".(get_option('cpg_div_size_x')-20)."px; height: 20px; cursor: hand;\"/><a href=\"".str_replace('normal_','',_cpg_get_image_uri($image))."\"><img src=\"./wp-content/plugins/coppermine/img/lens.gif\" alt=\"$title\" title=\"Fullsize: $title\" style=\"display: none; border: 0px; margin: 2px; width: 16px; height: 16px; cursor: hand;\"/></a>";
		    
		    $html="<div onmouseout=\"javascript:this.getElementsByTagName('img')[3].style.display = 'none';\" onmouseover=\"javascript:this.getElementsByTagName('img')[3].style.display = '';\" style=\"width: ".get_option('cpg_div_size_x')."px; height: ".get_option('cpg_div_size_y')."px;\" id=\"cpg-image\"><div style=\"width: ".get_option('cpg_div_size_x')."px; height: ".get_option('cpg_div_size_y')."px; background: url($uri) ".get_option('cpg_div_options')."\">".$lens."</div></div>";
		}
	}
	else {
		// full-scale images
		$uri = _cpg_get_image_uri($image);
#		if(get_option('cpg_image_style')==0) {
		    $html="<img src=\"$uri\" alt=\"$title\" title=\"$title\" {$class}{$extra_html}/>";
#		} else {
#		    $html="<div style=\"width: ".get_option('cpg_div_size_x')."px; height: ".get_option('cpg_div_size_y')."px;\" id=\"cpg-image\"><div style=\"width: ".get_option('cpg_div_size_x')."px; height: ".get_option('cpg_div_size_y')."px; background: url($uri) ".get_option('cpg_div_options')."\"><img src=\"http://images.itgweb.de/images/spacer.gif\" alt=\"$title\" title=\"$title\" style=\"width: ".get_option('cpg_div_size_x')."px; height: ".get_option('cpg_div_size_y')."px; cursor: hand;\"/></div></div>";
#		}

	}

	// put in link
	if (!$inhibit_link) { // && isset($options['link']) && !empty($options[$link])) {
		$html = _cpg_link(
			$html,
			$image,
			$options['link']);
	}

	if (!$options['bare']) {
		// add div
		if ($title_style!='inline') {
			$html = "<div>$html</div>";
		}

		if (empty($title) && !$style['in_group']) {
//			$title='&nbsp;';
		}
		
		// add label
		if (!$inhibit_link) { // && isset($options['link']) && !empty($options[$link])) {
			$title = _cpg_link(
				$title,
				$image,
				$options['link']);
		}
	
		// display title
		if(get_option('cpg_show_filename'))
		switch($title_style) {
			case 'pre':
				$html = "<div>$title</div>".$html;
				break;
			case 'inline':
				$html.="$title";
				break;
			default:
				$html .= "<div class=\"cpg-label\">$title</div>";
		}

		// add shadow
		if (!$inhibit_shadow && (
				(isset($options['shadow']) && $options['shadow']) ||
				(!isset($options['shadow']) && get_option('cpg_use_shadows'))
			))
		{
			$html = _cpg_shadow($html);
		}

	}

	if (!$options['bare']) {
		// put in aligning div
		if (!isset($options['in_group']) && (
			(isset($options['align']) && !empty($options['align'])) ||
			cpg_is_image($image) || 
			isset($options['thumb'])) )
		{
			$html=_cpg_fmt($html,$options['align']);
		} 

		if (!isset($options['in_group'])) {
			$str = _cpg_fmt_p($str);
		}
	}
		
	return $html;
}

function cpg_format_group($images, $style)
{
	$bare=false;

	$img_options = array(
//		'thumb' => true,
		'class' => 'cpg-img-box',
//		'link'  => $link_style,
//		'align' => $alignment,
		'align' => 'l',
		'shadow' => false,
		'in_group' => true
	);

	$img_override = array();
	
	if (!isset($style['class'])) {
		switch($style['align']) {
			case 'l':
				$class='cpg-grp-left';
				break;
			case 'r':
				$class='cpg-grp-right';
				break;
			case 'bl':
				$class='cpg-grp-leftbar';
				$img_override['bare']=true;
				break;
			case 'br':
				$class='cpg-grp-rightbar';
				break;
			case 'h':
				$class='cpg-grp-bar';
				$img_override['bare']=true;
				break;
			case 's':
				$class='slideshow';
				$img_override['bare']=true;
				break;
			case 'cb':
				$class='cpg-centerbar';
				break;
			case '':
			case 'c':
			default:
				$class='cpg-grp-center';
				$img_override['bare']=true;
				break;
		}
	} else {
		$class=$style['class'];
	}


/*	// start container div
	$html = "<div style=\"float:left;\">";

	// and an ugly workaround for ie
	$html .= '<div class="clearer"></div>';
*/
	foreach($images as $imagedef) {
		$image=$imagedef[0];
		$istyle=$imagedef[1];

		cpg_default_style_set($istyle, $img_options);
		cpg_override_style_set($istyle, $img_override);

/*		if (!$istyle['bare']) {
			$html.='<div class="cpg-img-box">';
		}
*/
		$float = $istyle['align'];
		if($float=='l') {
			$float = 'left';
		}
		if($float=='c') {
			$float = 'center';
		}
		if($float=='r') {
			$float = 'right';
		}
		$html.='<div style="float: '.$float.'">'.cpg_format_image($image, $istyle).'</div>';

/*		if (!$istyle['bare']) {
			$html.='</div>';
		}
*/
	}
/*
	//$html.='</div>';
	$html .= '<div class="clearer">&nbsp;</div>';

	// add title
	if (isset($style['title'])) {
		$html.='<div class="cpg-album-link">';
		if (isset($style['link'])) {
			$html .= '<a href="'.$style['link'].'">'.$style['title'].'</a>';
		} else {
			$html .= $style['title'];
		}
		$html.='</div>';
	}
	
	// close divs
	$html .= '</div>';
*/
	// add shadow
	if ($style['shadow']) {
		$html = _cpg_shadow($html);
	}
	
	
	return $html;
}


/*----------------------------------------------------------------------------*/
/*								 GET FUNCTIONS                                */
/*----------------------------------------------------------------------------*/

/*
 * The following functions return a piece of HTML, but don't display anything.
 * If the Database is offline, they should return the empty string.
 *
 * See section 'Template Tags' below if you want to use this plugin in your
 * themes.
 */

function cpg_get_album_wrap($aid, $num=0, $str_opts)
{
	$opts = _cpg_explode_group_args($str_opts);
	return cpg_get_album($aid, $num, false, $opts[0], $opts[1]);
}
 
// CREATE AN ALBUM div
// params: aid, number of pictures, random order
function cpg_get_album($aid, $num=0, $random=false, $style_opts=array(), $img_style_opts=array())
{
	global $cpg_base;
	global $cpg_uri;

	if ($num==1 && $random==false) {
		$thumb = cpg_db_get_album_thumbnail($aid);
		if (!empty($thumb)) {
			$dbimages = array(cpg_db_get_imagedata($thumb));
		}
	}
	else {
		$dbimages = cpg_db_get_album_images($aid,$num,$random);
	}
		
	// album meta data
	$info = cpg_db_get_albumdata($aid);
	if (!isset($style_opts['title'])) {
		$style_opts['title'] = $info->title;
	}

	if (!isset($style_opts['link'])) {
		$style_opts['link'] = $cpg_base.'thumbnails.php?album='.$aid;
	}
	
	// display nothing if there are no images
	if (!$dbimages) {
		return '';
	}

	$img_defaults = array(
		'thumb' => true,
		'class' => 'cpg-album-image',
		'align' => 'l',
		'shadow' => false,
		'in_group' => 1,
	);

	cpg_default_style_set($img_style_opts, $img_defaults);

	$images = array();
	foreach($dbimages as $dbimage) {
		array_push($images, array($dbimage, $img_style_opts));
	}
	$html = cpg_format_group($images, $style_opts);

	return $html;
}


// return an image div
function cpg_get_image($aid,$pos,$style_opts=array())
{
	cpg_default_style($style_opts, 'class', 'cpg-image-normal');

	$image = cpg_db_get_image($aid,$pos);
	$html = cpg_format_image($image, $style_opts);
	return $html;
}

// return an image
function cpg_get_image_absolute($pid,$style_opts=array()) 
{
	cpg_default_style($style_opts, 'class', 'cpg-image-normal');

	$image = cpg_db_get_imagedata($pid);
	$html = cpg_format_image($image, $style_opts);
	return $html;
}

// return an thumbnail div
function cpg_get_thumbnail($aid,$pos,$style_opts=array())
{
	cpg_default_style($style_opts, 'class', 'cpg-image-thumb');
	cpg_override_style($style_opts, 'thumb', true);

	$image = cpg_db_get_image($aid,$pos);
	$html = cpg_format_image($image, $style_opts);
	return $html;
}


// return a thumbnail div
function cpg_get_thumbnail_absolute($pid,$style_opts=array())
{
	cpg_default_style($style_opts, 'class', 'cpg-image-thumb');
	cpg_override_style($style_opts, 'thumb', true);
	
	$image = cpg_db_get_imagedata($pid);
	$html = cpg_format_image($image, $style_opts);
	return $html;
}

// get a series of images as thumbnails in a div.
// parameters:
// $number:   number of images to display.
// $owner_id: owner, 0 means all owners.
// $random:   last images if false, random images if true.
// $divclass: CSS class for the containing div.
// $imgclass: CSS class for the images.
function cpg_get_images($num, $owner_id=0, $random=false, $link_style='', $style_opts=array())
{
	$divclass='cpg-last-images';
   	$imgclass='cpg-last-image';

	global $cpg_base;
	global $cpg_uri;

	if ($random) {
		$images = cpg_db_get_random_images($num, $owner_id);
	} else {
		$images = cpg_db_get_last_images($num,$owner_id);
	}

	// make html mini album
	if (!$images) {
		return '';
	}

	$html = "<div class=\"$divclass\">";

	$img_options = array(
		'class'	=> $imgclass,
		'thumb'	=> true,
		'link'  => $link_style,
	);

	cpg_override_style_set($img_options, $style_opts);

	// add images
	$pos=0;
	foreach($images as $image) {
		$html .= cpg_format_image($image, $img_options);
		$pos++;
	}

	$html .= '</div>';

	return $html;
}

/* return last $num images */
function cpg_get_last_images($num, $owner_id=0,$style_opts=array())
{
	return cpg_get_images($num, $owner_id, false,'',$style_opts);
}

/* return random images from gallery */
function cpg_get_random_images($num, $owner_id=0,$style_opts=array())
{
	return cpg_get_images($num, $owner_id, true,'',$style_opts);
}

/* return a link to the current user's gallery. if a user is logged in. */
function cpg_get_user_gallery_link($pre="<div id=\"cpg-user-gallery-link\">", $post="</div>")
{
	global $user_ID;
	global $cpg_base;
	global $cpg_cmdb;

	if ($cpg_cmdb->unavailable) {
		return;
	}
	
	$wp_user = get_userdata($user_ID);
	$cpg_user = cpg_get_login_info($wp_user->user_login);
	if ($cpg_user->user_id>0) {
		$html = $pre . "<a href=\"$cpg_base" . "index.php?cat=" . (10000+$cpg_user->user_id) . "\">My Albums</a>" . $post;
	}

	return $html;
}

/* return a link to an album */
function cpg_get_album_link($aid, $title, $pre="<div id=\"cpg-user-album-link\">", $post="</div>")
{
	global $cpg_base;
	global $cpg_cmdb;

	if ($cpg_cmdb->unavailable) {
		return;
	}

	return $pre.'<a href="'.$cpg_base.'thumbnails.php?album='.$aid.'" title="'.$title.'">'.$title.'</a>'.$post;
}

/* return a link to the main gallery page */
function cpg_get_index_page_link($pre='', $post='')
{
	global $cpg_base;
	global $cpg_cmdb;
	if ($cpg_cmdb->unavailable) {
		return;
	}
	
	return $pre."<a href=\"$cpg_base\" title=\"Gallery Index Page\">Albums Index</a>".$post;
}

/*
 * return a link to the upload section, uses option cpg_popup_upload
 */
function cpg_get_upload_link($pre='', $post='') {
	global $user_ID;
	global $cpg_base;
	global $cpg_cmdb;
	if ($cpg_cmdb->unavailable) {
		return;
	}

	$html = $pre;
	$uri=$cpg_base.'wp-upload.php';
	if (intval(get_option('cpg_popup_upload'))) 
	{
		if (intval(get_option('cpg_integrate_users'))) {
			$wp_user = get_userdata($user_ID);
			if ($wp_user) {
				$cpg_user = cpg_get_login_info($wp_user->user_login);
			}
			if ($cpg_user) {
				$uri.='?user='.$cpg_user->user_id.'&amp;pass='.$cpg_user->user_password_md5;
			}
		} else if (get_option('cpg_upload_user')) {
			$cpg_user = cpg_get_login_info(get_option('cpg_upload_user'));
			if ($cpg_user) {
				$uri.='?user='.$cpg_user->user_id.'&amp;pass='.md5(get_option('cpg_upload_pass'));
			}
		}

		$html.="<input class=\"cpg-action-button\" type=\"button\" onclick=\"window.open('$uri','upload','scrollbars=yes,width=600,height=480,left=100,top=150,status=yes')\" value=\"Upload Files\" />";
	} 
	else {
		$html.="<input class=\"cpg-action-button\" type=\"button\" onclick=\"document.location='$uri'\" value=\"Upload Files\">";
	}
	$html.=$post;
	return $html;
}

/*
 * return a link to the album manager popup, for use with 
 * the upload popup.
 */
function cpg_get_album_manager_link($pre='', $post='')
{
	global $cpg_base;
	
	$html=$pre;
	$uri=$cpg_base.'wp-albmgr.php';
	$html.="<a href=\"javascript:void(window.open('$uri','upload','scrollbars=yes,width=600,height=480,left=100,top=150,status=yes'))\" title=\"Manage Albums\">".__('Manage Albums').'</a>';
	$html.=$post;
	return $html;
}

/*
 * return a clearing element to force the next image to be
 * on a different line
 */
function cpg_get_newline()
{
	return '<div class="clearer">&nbsp;</div>';
	
}


/* 
 * return comments for an image 
 */
function cpg_get_image_comments($pid, $num=5)
{
	$comments = cpg_db_get_image_comments($pid, $num);

	if (!$comments || !count($comments)) {
		return '';
	}	
	
	$html='<ul class="cpg-comments">';

	foreach($comments as $comment) {
		$uri = _cpg_get_image_uri($image);
		$author = htmlspecialchars($comment->msg_author);
		$date = mysql2date( get_settings('date_format'));
		$html.="<li><span class=\"date\">$date</span>: <span class=\"author\">$author</span> commented on <a href=\"$url\">a picture</a>. </li>";
	}
		
	$html.='</ul>';

	return $html;
}

/*
 * return recent comments 
 */
function cpg_get_recent_comments($num = 5)
{
	$comments = cpg_db_get_recent_comments($num);

	if (!$comments || !count($comments)) {
		return '';
	}
	
	$html='<ul class="cpg-comments">';
	foreach($comments as $comment) {
		$uri = _cpg_get_image_uri($image);
		$author = htmlspecialchars($comment->msg_author);
		$date = mysql2date( get_settings('date_format'));
		$html.="<li><span class=\"date\">$date</span>: <span class=\"author\">$author</span> commented on <a href=\"$url\">a picture</a>. </li>";
	}
	$html.='</ul>';

	return $html;
}

 


/*----------------------------------------------------------------------------*/
/*								 TEMPLATE TAGS                                */
/*----------------------------------------------------------------------------*/

/*
 * Insert a div containing $num images of album $aid.
 * If $random, take them at random, else take the most recent ones.
 */
function cpg_album($aid, $num=0, $random=false, $link_style='') 
{
	print cpg_get_album($aid,$num,$random,$link_style);
}

/*
 * Insert image at $pos from album $aid
 */
function cpg_image($aid,$pos,$link_style='')
{
	print cpg_get_image($aid,$pos,$link_stlye);
}

/*
 * Insert image with absolute Id $pid
 */
function cpg_image_absolute($pid,$link_style='')
{
	print cpg_get_image_absolute($pid,$link_style);
}

/*
 * Insert thumbnail at $pos from album $aid
 */
function cpg_thumbnail($aid,$pos,$link_style='')
{
	print cpg_get_thumbnail($aid,$pos,$link_style);
}

/*
 * Insert thumbnail from image with absolute Id $pid
 */
function cpg_thumbnail_absolute($pid,$link_style='')
{
	print cpg_get_thumbnail_absolute($pid,$link_style);
}

/*
 * Insert $num most recent images belonging to $owner_id
 * if $owner_id is 0, choose from everyone's images.
 */
function cpg_last_images($num, $owner_id=0,$style=array())
{
	print cpg_get_last_images($num,$owner_id,$style);
}

/*
 * Insert num images belonging to $owner_id, chosen at random.
 * if $owner_id is 0, choose from everyone's images.
 */
function cpg_random_images($num, $owner_id=0,$style=array())
{
	print cpg_get_random_images($num,$owner_id,$style);
}


/*
 * Insert a clearing element to force the next image to be
 * on a different line
 */
function cpg_newline() 
{
	print cpg_get_newline();
}

/*
 * Insert a link to the current user's gallery.
 * If there is no logged in user, insert nothing.
 */
function cpg_user_gallery_link($pre="<div id=\"cpg-user-gallery-link\">", $post="</div>")
{
	print cpg_get_user_gallery_link($pre,$post);
}

/*
 * Insert a link to album $aid, optionally with $title.
 * If the album could not be found, insert nothing.
 */
function cpg_album_link($aid, $title='') 
{
	global $cpg_cmdb;
	if ($cpg_cmdb->unavailable) {
		return;
	}

	if ($title=='') {
		$info = cpg_db_get_albumdata($aid);
		if ($info) {
			$title=$info->title;
		}
	}
			
	if ($title) {
		print cpg_get_album_link($aid,$title);
	}
}

/*
 * Insert a link to CPG's index page.
 */
function cpg_index_page_link($pre='', $post='')
{
	print cpg_get_index_page_link($pre,$post);
}

/*
 * Insert a link to CPG's upload section
 *
 * Only display link if user has an album.
 * 
 * If option cpg_popup_upload is set, pressing the link will open a popup,
 * else it will just go to CPG's upload page.
 */
function cpg_upload_link($pre='', $post='')
{
	global $cpg_base;
	global $user_ID;

	global $cpg_cmdb;
	if ($cpg_cmdb->unavailable) {
		return;
	}
	
	if(!isset($user_ID)) {
		return;
	}
	
	$user=get_userdata($user_ID);
	if (!function_exists('get_role')) {
		// wp 1.5
		if ($user->user_level < get_option('cpg_min_level')) {
			return;
		}
	} else {
		// wp 1.6
		// Behaviour changed: always display upload button if 
		// inserter is shown
	}

	echo cpg_get_upload_link($pre,$post);
}


/******************************************************************************/
/*								 ADMIN SECTION                                */
/******************************************************************************/								 

/*----------------------------------------------------------------------------*/
/*									 LINKS                                    */
/*----------------------------------------------------------------------------*/

/*
 * insert a link to the upload section
 */
function _cpg_insert_upload_link($rows)
{
	cpg_upload_link('<div class="cpg-upload-link">', '</div>');
	return $rows;
}

/*
 * Insert a link to the Image selecter popup
 */
function cpg_selecter_link($pre='', $post='')
{
	global $cpg_cmdb;
	global $user_ID;

	if ($cpg_cmdb->unavailable) {
		return;
	}
	
	if(!isset($user_ID)) {
		return;
	}
	
	$user=get_userdata($user_ID);
	if (!function_exists('get_role')) {
		// wp 1.5
		if ($user->user_level < get_option('cpg_min_level')) {
			return;
		}
	} else {
		// wp 1.6
		// Behaviour changed: always display browser button if 
		// inserter is shown
	}
	

	$uri = get_settings('siteurl') . '/wp-content/plugins/coppermine/coppermine-browser.php';
	print $pre;
	print '<input class="cpg-action-button" type="button" onclick="window.open(\''.$uri.'\',\'chooser\',\'scrollbars=yes,width=620,height=500,left=100,top=150,status=yes\')" value="More Pictures"/>';
//	print "<a href=\"javascript:void()\" title=\"Choose from more pictures\">".__('More Pictures').'</a>';
	print $post;
}

/*
 * insert a link to the popup image selecter
 */
function _cpg_insert_image_selecter_link($rows)
{
	cpg_selecter_link('<div class="cpg-upload-link">', '</div>');
	return $rows;
}



/*----------------------------------------------------------------------------*/
/*								 IMAGE INSERTER                               */
/*----------------------------------------------------------------------------*/								 

/*
 * The following functions handle the admin section in WP.
 * They are used to
 * - Create the 1-click insert bar
 * - Create a link to CPG's upload section.
 * - Insert a linkt to the javascript needed for all this.
 */
	

/*
 * insert the album chooser
 */
function _cpg_image_insert_album_dropdown($user_id)
{
	$albums = cpg_db_get_albums($user_id);

?>  
	<li class="cpg-dropdown">
	  <span class="nobreak">
	  <label for="cpg-image-insert-select-album-number">Insert </label>
	  <select id="cpg-image-insert-select-album-number">
	  	<option value="0">All</option>
	    <option value="1">1</option>
	    <option value="2">2</option>
	    <option value="3">3</option>
	    <option value="4">4</option>
	    <option value="5">5</option>
	    <option value="6">6</option>
	    <option value="7">7</option>
	    <option value="8">8</option>
	    <option value="9">9</option>
	    <option value="10">10</option>
	  </select>
	  </span>
	</li>

	<li class="cpg-dropdown">
      <span class="nobreak">
	  <label for="cpg-image-insert-select-album">Images from Album: </label>
	  <select id="cpg-image-insert-select-album">
	    <option value="" selected="selected">-- Select One --</option>
<?php
	  foreach($albums as $album) {
		  print "<option value=\"$album->aid\" title=\"$album->description\">$album->title</option>";
	  }
?>
	  </select>
	  </span>
	</li>

	<li class="cpg-dropdown">
	<input type="button" onclick="cpg_insert_album();" value="Go!"/>
	</li>
<?
}

function _cpg_image_insert_shadow_dropdown()
{
?>
	<li class="cpg-dropdown">
	<span class="nobreak">
	<label for="cpg-image-insert-select-shadow">Shadow: </label>
	<select id="cpg-image-insert-select-shadow">
		<option value="" selected="selected">Default</option>
		<option value="1">Enable</option>
		<option value="0">Disable</option>
	</select>
	</span>
	</li>
<?php
}

function _cpg_image_insert_linktype_dropdown()
{
?>
	<li class="cpg-dropdown">
	<span class="nobreak">
	<label for="cpg-image-insert-select-linktype">Link: </label>
	<select id="cpg-image-insert-select-linktype">
		<option value="" selected="selected">Default</option>
		<option value="g">To Gallery</option>
		<option value="x">No link</option>
		<option value="p">Popup</option>
		<option value="d">Direct</option>
		<option value="<?php print get_option('siteurl'); ?>">Url</option>
	</select>
	</span>
	</li>
<?php
}

function _cpg_image_insert_condition_dropdown()
{
?>
	<li class="cpg-dropdown">
	<span class="nobreak">
	<label for="cpg-image-insert-select-condition">Condition: </label>
	<select id="cpg-image-insert-select-condition">
		<option value="" selected="selected">Always</option>
		<option value="single">Single</option>
		<option value="!single">Except single</option>
		<option value="excerpt">Excerpt</option>
		<option value="!excerpt">Direct</option>
		<option value="home">Index</option>
		<option value="!home">Not Index</option>
	</select>
	</span>
	</li>
<?php
}

function _cpg_image_insert_imagesize_dropdown()
{
?>
	<li class="cpg-dropdown">
	<span class="nobreak">
	<label for="cpg-image-insert-select-type">Image size: </label>
	<select id="cpg-image-insert-select-type">
      <option value="thumbnail">Thumbnail</option>
      <option value="image">Image</option>
    </select>
	</span>
	</li>
<?php
}

function _cpg_image_insert_imagestyle_dropdown()
{
	global $cpg_alignments;
?>
	<li class="cpg-dropdown">
	<span class="nobreak">
	<label for="cpg-image-insert-select-style">Alignment: </label>
	<select id="cpg-image-insert-select-style">
<?php
	$default=' selected="selected"';
	foreach ($cpg_alignments as $key => $fmt) {
		echo "<option value=\"$key\"$default>$fmt[0]</option>";
		$default='';
	}
?>
    </select>
	</span>
	</li>
<?php
}

function _cpg_image_insert_groupstyle_dropdown()
{
?>
	<li class="cpg-dropdown">
	<span class="nobreak">
	<label for="cpg-image-insert-gs">Style: </label>
	<select id="cpg-image-insert-gs">
		<option value="c">Center</option>
		<option value="l">Left Align</option>
		<option value="r">Right Align</option>
		<option value="bl">Left Bar</option>
		<option value="br">Right Bar</option>
		<option value="h">Horizontal Scroller</option>
		<?php /*		<option value="s">Slide Show</option> */ ?>
    </select>
	</span>
	</li>
<?php
}

function _cpg_image_insert_groupshadow_dropdown()
{
?>
	<li class="cpg-dropdown">
	<span class="nobreak">
	<label for="cpg-image-insert-gsd">Shadow: </label>
	<select id="cpg-image-insert-gsd">
		<option value="">Default</option>
		<option value="1">Enable</option>
		<option value="0">Disable</option>
    </select>
	</span>
	</li>
<?php
}

function _cpg_image_insert_groupcondition_dropdown()
{
?>
	<li class="cpg-dropdown">
	<span class="nobreak">
	<label for="cpg-image-insert-gcd">Condition: </label>
	<select id="cpg-image-insert-gcd">
		<option value="" selected="selected">Always</option>
		<option value="single">Single</option>
		<option value="!single">Except single</option>
		<option value="excerpt">Excerpt</option>
		<option value="!excerpt">Direct</option>
		<option value="home">Index</option>
		<option value="!home">Not Index</option>
    </select>
	</span>
	</li>
<?php
}


/*
 * Insert the dropdowns above the editor
 */
function _cpg_image_insert_dropdowns()
{
	echo '<ul id="cpg-image-insert-options" class="cpg-dropdown-bar">';

	_cpg_image_insert_imagesize_dropdown();
	_cpg_image_insert_imagestyle_dropdown();

	_cpg_image_insert_shadow_dropdown();
	_cpg_image_insert_linktype_dropdown();
	//_cpg_image_insert_condition_dropdown();

	echo '<li class="clearer">&nbsp;</li>';

	echo '</ul>';
}

/*
 * Insert album/group dropdowns
 */
function _cpg_image_insert_group_dropdowns($user_id)
{
echo <<<HTML
	<ul id="cpg-groups-bar" class="cpg-dropdown-bar" style="display:none;">
		<li>
			<ul>
HTML;

   	_cpg_image_insert_groupstyle_dropdown(); 
	_cpg_image_insert_groupshadow_dropdown(); 
	//_cpg_image_insert_groupcondition_dropdown(); 

echo <<<HTML
			</ul>
		</li>
		<li class="clearer">&nbsp;</li>
		<li>
			<ul>
HTML;

	_cpg_image_insert_album_dropdown($user_id); 

echo <<<HTML
			</ul>
		</li>
		<li class="clearer">&nbsp;</li>
		<li>
			<ul>
				<li class="cpg-dropdown">Insert group tag: <input type="button" onclick="cpg_insert_grouptag()" value="Group"/></li>

			</ul>
		</li>

	<li class="clearer">&nbsp;</li>

	</ul>
HTML;
}


/*
 * Insert the one-click inserter bar:
 * - dropdown menus
 * - more recent pictures
 */
function _cpg_image_inserter($rows)
{
	cpg_image_inserter();
	return $rows;
}
 
function cpg_image_inserter()
{
	global $user_ID;
	global $cpg_base;
	global $cpg_uri;

	// default to images from all users
	$cpg_user_id = 0;
	
	if(!isset($user_ID)) {
		return;
	}

	$user=get_userdata($user_ID);
	if (!function_exists('get_role')) {
		// wp 1.5
		if ($user->user_level < get_option('cpg_min_level')) {
			return;
		}
	}
	else {
		// wp 1.6/2.0
		if (!current_user_can('cpg_display_inserter')) {
			return;
		}
	}

	if (intval(get_option('cpg_integrate_users'))) {
		// user-integration enabled, only fetch own pictures
		$cpg_user=cpg_get_login_info($user->user_login);
		if (!$cpg_user->user_name) {
			// no coppermine account
			return;
		}

		$cpg_user_id = $cpg_user->user_id;
	} 	

	$num_insert = intval(get_option('cpg_inserter_num'));
	
	if(intval(get_option('cpg_upload_user_id')))
	{
		$cpg_user_id = get_option('cpg_upload_user_id');
	}
	
	$images = cpg_db_get_last_images($num_insert, $cpg_user_id);

	print '<input type="button" id="cpg-show-div" style="display: none; margin-left: 0px;" value="Show Coppermine Inserter +" onclick="cpg_showdiv_but(this, \'cpg-image-inserter\')"></input>';

	print 
  	  '<div id="cpg-image-inserter">';

	// buttons
	print '<ul id="cpg-button-bar">';

	print '<li class="left-button"><input type="button" value="Hide" onclick="cpg_hidediv_but(this, \'cpg-image-inserter\', \'cpg-show-div\')"></input></li>';

	if (intval(get_option('cpg_upload_link'))) {
		cpg_upload_link('<li class="left-button">','</li>');
	}

	if (intval(get_option('cpg_selecter_link'))) {
		 cpg_selecter_link('<li class="left-button">','</li>');
	}

	echo '<li><input class="cpg-collapse" type="button" onclick="cpg_togglediv_but(this, \'cpg-image-bar\')" value="Images" /></li>';
	echo '<li><input class="cpg-expand" type="button" onclick="cpg_togglediv_but(this, \'cpg-groups-bar\')" value="Groups" /></li>';
	echo '<li><input class="cpg-collapse" type="button" onclick="cpg_togglediv_but(this, \'cpg-image-insert-options\')" value="Options" /></li>';

	print '</ul>';
	print '<div class="clearer">&nbsp;</div>';

	_cpg_image_insert_dropdowns();
	_cpg_image_insert_group_dropdowns($cpg_user_id);

	$count=count($images);

	print '<ul id="cpg-image-bar">';

	// insert empty items in front if not enough images
	for ($i=0; $i<$num_insert-$count; $i++) {
		print '<li id="insert-image-'.$i.'" class="cpg-inserter-image"></li>';
	}

	// add images
	if ($images) foreach($images as $image) {
		print '<li class="cpg-inserter-image" id="insert-image-'.$i++.'">';
		$uri = _cpg_get_thumb_uri($image);
		$title = "Insert {$image->title} ({$image->filename})";
		print 
		"  <img src=\"$uri\" alt=\"$title\" title=\"$title\" onclick=\"cpg_insert_image('".$image->pid."')\" /> ";
		print '</li>';
	}

	// add 'newline' element
	$newlineimg= get_settings('siteurl') . '/wp-content/plugins/coppermine/img/newline.png';
	print '<li class="cpg-inserter-image"><img src="'.$newlineimg.'" title="Insert newline tag" alt="Insert newline tag" onclick="cpg_insert_nl();" /></li>';

	print '</ul>';

	print '<div class="clearer">&nbsp;</div>';
	
	print '</div>';

	return;
}

/* 
 * Insert some javascript required to make image insertion work. It depends on
 * the edInsertContent from wp-admin/quicktags.js . 
 */
function _cpg_image_inserter_head()
{
?>
<script type="text/javascript" src="<?php echo get_settings('siteurl'); ?>/wp-content/plugins/coppermine/coppermine.js" ></script>

<script type="text/javascript">
	var cpg_inserter_num=<?php echo get_option('cpg_inserter_num'); ?>;
</script>
<?php
}

/******************************************************************************/
/*								USER INTEGRATION                              */
/******************************************************************************/

/* 
 * on userlogin, login to coppermine (by setting a cookie)
 * if the user doesn't exist in the coppermine user database, 
 * create her from wordpress login info.
 * 
 * details:
 * - login by setting a coppermine cookie.
 * - users need a level >= cpg_user_min_level_create  to have get a coppermine account.
 * - auto-create user if she doesn't exist.
 * - add the user to the default group. (customizable in this file)
 * 
 * issues: 
 * - passwords: 
 *   + coppermine stores passwords in clear text, wordpress as md5 hashes. A
 *     patch needs to be applied to the CPG sources before you can use user integration.
 * - logins:
 *   + coppermine uses 25 chars, wordpress 65. Logins will be silently truncated.
 */


/*
 * Login to cpg
 */
function cpg_login($login)
{
	global $cpg_base;
	global $CPG_CONFIG;

	$wp_user = get_userdatabylogin($login); 

	if (function_exists('get_role')) {
		// wp 1.6/2.0 whatever
		$nwu = new WP_User($login);
		if (!$nwu->has_cap('cpg_auto-login')) {
			return;
		}
	}
	else {
		// wp 1.5
		if ( $wp_user && $wp_user->user_level < get_option('cpg_user_min_level_create') ) {
			return;
		}
	}

	cpg_connect();
	$cpg_user = cpg_get_login_info($login);

	if (!$cpg_user) {
		// create user
		if (intval(get_option('cpg_auto_create_users'))) {
			cpg_import_user($wp_user);
			$cpg_user = cpg_get_login_info($login);
		}
	} 

	if ($cpg_user) {
		if (intval(get_option('cpg_auto_create_album'))) {
			// create default gal
			cpg_create_default_album($cpg_user);
		}

		if ($CPG_CONFIG['version']>=140) {
			_cpg_remote_login_140($cpg_user);
		}
		else {
			if (intval(get_option('cpg_remote_login'))) {
				_cpg_remote_login_130($cpg_user);
			}
			else {
				// set local cookie 
				$until = time() + 86400 * 30;
				setcookie($CPG_CONFIG['cookie_name'] . '_uid', $cpg_user->user_id, $until, $CPG_CONFIG['cookie_path'] );
				setcookie($CPG_CONFIG['cookie_name'] . '_pass', $cpg_user->user_password_md5, $until, $CPG_CONFIG['cookie_path'] );
			}
		}
		
		// update last visit time
		cpg_touch_user($cpg_user->user_id);
	}
}

function _cpg_remote_login_130($cpg_user)
{
	global $cpg_base;
	
	// use login page on remote server and set it up to come back
	global $redirect_to;
	if (empty($redirect_to)) {
		if (isset($_POST['redirect_to'])) {
			$redirect_to = preg_replace('|[^a-z0-9-~+_.?#=&;,/:]|i', '', $_POST['redirect_to']);
		} else {
			$redirect_to=get_settings('siteurl').'/wp-admin/';
			if ($user->user_level < 2) {
				$redirect_to=get_settings('siteurl').'/wp-admin/profile.php';
			}
		}
	} else {
		if (!preg_match('|^http:?//|',$redirect_to)) {
			$redirect_to=get_settings('siteurl').'/'.$redirect_to;
		}
	}
	$referrer=cpg_urlencode($redirect_to);

	wp_redirect("{$cpg_base}wp-login.php?user={$cpg_user->user_id}&pass={$cpg_user->user_password_md5}&referrer=$referrer");

	exit;
}

function _cpg_remote_login_140($cpg_user)
{
	global $CPG_CONFIG;
	global $cpg_base;
	global $cpg_cmdb;
	
	// generate client id
	$client_id = md5($_SERVER['HTTP_USER_AGENT'].$_SERVER['SERVER_PROTOCOL'].$CPG_CONFIG['ecards_more_pic_target']);

	// generate a session id
	$failsafe = 20;
	$randnum = 0;
	while ($failsafe--) {
		$randnum = md5( uniqid( microtime(), 1 ));
		$session_id = $randnum.$client_id;

		$sql = "SELECT count(session_id) FROM ".get_option('cpg_table_prefix').'sessions'." WHERE session_id=MD5('$session_id')";
		if ($cpg_cmdb->get_var($sql)==0) {
			break;
		}
	}

	// create session entry in coppermine sessions table
	$sql =  'insert into '.get_option('cpg_table_prefix').'sessions '.
	        '(session_id, user_id, time, remember) values '.
			'("'.md5($session_id).'", '.intval($cpg_user->user_id).', "'.time().'", 0);';
	$cpg_cmdb->query($sql);

	// determine where to redirect after remote login
	global $redirect_to;
	if (empty($redirect_to)) {
		if (isset($_POST['redirect_to'])) {
			$redirect_to = preg_replace('|[^a-z0-9-~+_.?#=&;,/:]|i', '', $_POST['redirect_to']);
		} else {
			$redirect_to=get_settings('siteurl').'/wp-admin/';
		}
	} else {
		if (!preg_match('|^http:?//|',$redirect_to)) {
			$redirect_to=get_settings('siteurl').'/'.$redirect_to;
		}
	}
	$referrer=cpg_urlencode($redirect_to);

	# redirect to wp-login.php page in Coppermine

	wp_redirect("{$cpg_base}wp-login.php?user={$cpg_user->user_id}&session=".$randnum."&referrer=$referrer");

	exit;
	
}

/*
 * .. Logout from CPG
 */
function cpg_logout($login)
{
	global $CPG_CONFIG;
	global $cpg_base;

	cpg_connect();

	if (intval(get_option('cpg_remote_login'))) {
		// open window on remote server
		$redirect_to = get_settings('siteurl').'/wp-login.php';
		$referrer=cpg_urlencode($redirect_to);
//		$referrer='http://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['PHP_SELF']).$redirect_to;
		wp_redirect("{$cpg_base}wp-login.php?action=logout&referrer=$referrer");
		exit;
	}
	else {
		// set local cookie 
		setcookie($CPG_CONFIG['cookie_name'] . '_pass', '', time()-31536000, $CPG_CONFIG['cookie_path'] );
		setcookie($CPG_CONFIG['cookie_name'] . '_uid', '', time()-31536000, $CPG_CONFIG['cookie_path'] );
		return;
	}
	
}

/******************************************************************************/
/*								CAPABILITIES MGT							  */
/******************************************************************************/

function cpg_add_caps($caps)
{
	if (!in_array('cpg_auto-login', $caps)) {
		$caps[] = 'cpg_auto-login';
	}
	if (!in_array('cpg_display_inserter', $caps)) {
		$caps[] = 'cpg_display_inserter';
	}
	return $caps;
}


/******************************************************************************/
/*									 GLOBAL                                   */
/******************************************************************************/

/*
 * int main(int argc, char** argv) { ...
 */

// return if not configured
if (!get_option('cpg_options_version'))
	return;

// connect to database early, but after html start when possible..
// until wp2.0 an event is missing in case of an rss, and even then you'd have
// to use rss_head, rss2_head atom_head and rdf_head.
// Also connection on cpg_filter now. This will not work in case of a PHP page 
// using template tags directly is used in a feed, but frankly, who does such a thing?
// and if she does, she'll probably know how to find this remark and include a cpg_connect
// in the file.
add_action('wp_login', 		'cpg_connect');
add_action('admin_head', 	'cpg_connect');
add_action('wp_head', 		'cpg_connect');
add_action('template_redirect', 'cpg_connect');

// quicktags
add_filter('the_content', 'cpg_filter' );
add_filter('the_content', 'cpg_de_p_er' );

// stylesheet
add_filter('wp_head', '_cpg_insert_stylesheet');
add_filter('wp_head', '_cpg_insert_javascript');

// login integration
if (intval(get_option('cpg_integrate_users')) && intval(get_option('cpg_auto_login'))) {
	add_action('wp_login',  'cpg_login');
	add_action('wp_logout', 'cpg_logout');
}	

// integrate with editor
if (strstr($_SERVER['PHP_SELF'], 'post-new.php') ||
    strstr($_SERVER['PHP_SELF'], 'post.php') ||
    strstr($_SERVER['PHP_SELF'], 'page.php') ||
    strstr($_SERVER['PHP_SELF'], 'page-new.php'))
{
	if (intval(get_option('cpg_use_inserter'))) {
		add_filter('admin_head', '_cpg_insert_stylesheet');
		add_filter('admin_head', '_cpg_image_inserter_head');
		add_action('option_default_post_edit_rows', '_cpg_image_inserter');
	}
}

// capabilities: integrate with Owen Winkler's Role Manager
add_filter('capabilities_list', cpg_add_caps);


/******************************************************************************/

/*

Based on version 0.1 of Matthias Jell's plugin who adapted Jason Goldsmith
(Unteins)'s plugin (http://www.texx.org). Matthias' advanced functionality was
backported in the current plugin. Sorry Matthias for changing your style so
much.


LICENSE
=======

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You can get the GNU General Public License online, at
http://www.gnu.org/copyleft/gpl.html, or you can write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


CHANGELOG
=========

Changes 3.3
- Removed fopen (old debugging statement)

Changes 3.4
- Centralized path generation functions
- escape paths
- Icons for other media types
- Dewplayer integration

Changes 3.5
- cpg_get_recent_comments
- cpg_get_image_comments
- changed CSS cpg-album-label to cpg-album-link
- groups: center, left, right, left/right bar, horizontal scroller, centered evenspace
- changed align option from 1 letter to any num of letters
- changed album tag to use groups internally
- use styles hash for last_images and random_images
- groups and albums can have default options for the contained elements after ::
- fixed bug: use number of pictures for album insert
- added 'd' link type (direct to resource)
- default to link to resource for non-picture files
- tag option to change class
- adapted the inserter bar to use some of the new attributes
- added a group options sub-bar to the inserter bar
- fixed 'no link' link style (l=x)
- created an explicit attribute for inline images (no divs, no title) ([image:i])
- make popups work with non-image styles
- changed cookie handling on logout in wp-login.php

Changes 3.6
- handle multiline groups
- allow fullsize images in groups
- make group button insert [group] tags around selection

Changes 3.7
- coppermine.js: compatible with tinyMCE and WP1.6
- sanitized insertbar (equal margins, don't split label+dropdown except with ie)
- added hide button to hide entire inserter
- display hide/collapse status of buttons with images
- removed 'dotted' CSS border attribute because ie thinks it's dashed
- use scrolling display for pictures in inserter bar
- Made an upload popup window for cpg 1.4
- remote login for cpg 1.4
- autodetect version
- upgraded Dewplayer
- import Coppermine users
- Don't display database upgrade options in configuration screens
- autocreate coppermine users on login if option is set
- Don't hide inserter bar when there are no images to display

Changes 3.8
- don't put DIVs in Ps
- css corrections for ie in leftbar group mode
- don't handle [album] with one image in a different way than with multiple images
- update image inserter after upload
- display icons in inserter instead of text labels for non images uploads
- fixed escaping problems in urls
- don't use &$ to pass args by references, function declaration is enough
- force width and height with w= and h=
- fix image and album insertion from browser
- don't submit when image browser is opened 

Changes 3.9
- dont use last_addition column in cpg_db_get_albums
- repaired cpg_can_access_album
- repaired privacy settings in cpg_db_search_albums cpg_db_search_pictures
- added cpg_connect to cpg_filter to have the engine ready in RSS streams.
- don't wonkify the inserter bar after an image upload
- correction of access rights for browser and inserter: you can always see you own
  pics now.
- Use capabilities for wp20 for auto-login and use-inserter
- changed configuration screen to use caps
- manage differences between cpg13 and cpg14 better
- Use caps with image browser

Changes 4.0
- new author :D
- added: lightbox2 integration (lightbox[my] for grouping)
- added: cpg_upload_user_id option for simple just working user integration (just a stupid hack, working on a better solution)
- added: support of wordpress version 2.xx
- added: option for showing/hiding title/filename under images (global setting)
- added: option for showing thumbnails in divs, so they can have all the same orientation (auto cropping, size can be changed in optiond)
- added: small lens icon for show the fullsize picture (only in div-view)
- changed: size of the imagebrowser to see the scrollbars
- changed: if you are not logged in, coppermine config will kick you to standard wordpress login page

// FIX image browser
  

// FEATURES		 
// TODO: tranparent shadows (not to white)
// TODO: add attribute to specify border class
// TODO: add slideshow for albums
// TODO: exclude non-picture files from random/last
// TODO: add cpg_tag('[image:blah]') function for templates
// TODO: [image-name], [image-description]
// TODO: tooltips with title+description
// TODO: bbcode for tooltips
// TODO: get_comments_merged? define template for comments? include title?
// TODO: support more than 1 shadows/border set (already possible with c=)

// API changes:
- cpg_get_album( $aid, $num, $random, $style_opts )
- cpg_get_image( $aid, $pos, $style_opts )
- cpg_get_image_absolute( $pid, $style_opts )
- cpg_get_thumbnail( $aid, $pos, $style_opts )
- cpg_get_thumbnail_absolute( $pid, $style_opts )
- cpg_get_random_images($num, $owner, $style_opts)  
- cpg_get_last_images($num, $owner, $style_opts)

// API new:
- cpg_get_images_comments($pid, $num)
- cpg_get_recent_comments($num)

IE BUGS ENCOUNTERED
-------------------

- with overflow: auto, ie doesn't change the height of a div when the scrollbar is
  shown, resulting in cut contents unless a 'safety margin is added'.
  workaround: 
    height: 100% !important;
	height: 150px;
	overflow: auto;
	overflow-x: auto;
	overflow-y: hidden;
  on ie there will be an extra margin for the scrollbar, on ff the div will be resized to
  contain the scrollbar
	
- when it's forbidden to break at whitespace, with white-space: nowrap, ie thinks there
  shouldn't be breaks between siblings either. 
  workaround: 
    wrap the elements in a <span style="white-space: nobreak"></span>
	set the style of the containing element (li for instance) to style="white-space: pre"
  this bug also affects opera

  MINT STREWN DARK JAVA / Last rig
*/

?>
